@@ -39,9 +39,9 @@ use crate::with_catalog_id;
 #[derive(Debug, TypedBuilder)]
 /// Glue Catalog configuration
 pub struct GlueCatalogConfig {
-    #[builder(default, setter(strip_option))]
+    #[builder(default, setter(strip_option(fallback = uri_opt)))]
     uri: Option<String>,
-    #[builder(default, setter(strip_option))]
+    #[builder(default, setter(strip_option(fallback = catalog_id_opt)))]
     catalog_id: Option<String>,
     warehouse: String,
     #[builder(default)]
@@ -355,7 +355,9 @@ impl Catalog for GlueCatalog {
             }
         };
 
-        let metadata = TableMetadataBuilder::from_table_creation(creation)?.build()?;
+        let metadata = TableMetadataBuilder::from_table_creation(creation)?
+            .build()?
+            .metadata;
         let metadata_location = create_metadata_location(&location, 0)?;
 
         self.file_io
@@ -381,14 +383,12 @@ impl Catalog for GlueCatalog {
 
         builder.send().await.map_err(from_aws_sdk_error)?;
 
-        let table = Table::builder()
+        Table::builder()
             .file_io(self.file_io())
             .metadata_location(metadata_location)
             .metadata(metadata)
             .identifier(TableIdent::new(NamespaceIdent::new(db_name), table_name))
-            .build();
-
-        Ok(table)
+            .build()
     }
 
     /// Loads a table from the Glue Catalog and constructs a `Table` object
@@ -432,7 +432,7 @@ impl Catalog for GlueCatalog {
                 let metadata_content = input_file.read().await?;
                 let metadata = serde_json::from_slice::<TableMetadata>(&metadata_content)?;
 
-                let table = Table::builder()
+                Table::builder()
                     .file_io(self.file_io())
                     .metadata_location(metadata_location)
                     .metadata(metadata)
@@ -440,9 +440,7 @@ impl Catalog for GlueCatalog {
                         NamespaceIdent::new(db_name),
                         table_name.to_owned(),
                     ))
-                    .build();
-
-                Ok(table)
+                    .build()
             }
         }
     }
