@@ -346,7 +346,9 @@ impl Catalog for HmsCatalog {
             }
         };
 
-        let metadata = TableMetadataBuilder::from_table_creation(creation)?.build()?;
+        let metadata = TableMetadataBuilder::from_table_creation(creation)?
+            .build()?
+            .metadata;
         let metadata_location = create_metadata_location(&location, 0)?;
 
         self.file_io
@@ -369,14 +371,12 @@ impl Catalog for HmsCatalog {
             .await
             .map_err(from_thrift_error)?;
 
-        let table = Table::builder()
+        Table::builder()
             .file_io(self.file_io())
             .metadata_location(metadata_location)
             .metadata(metadata)
             .identifier(TableIdent::new(NamespaceIdent::new(db_name), table_name))
-            .build();
-
-        Ok(table)
+            .build()
     }
 
     /// Loads a table from the Hive Metastore and constructs a `Table` object
@@ -407,7 +407,7 @@ impl Catalog for HmsCatalog {
         let metadata_content = self.file_io.new_input(&metadata_location)?.read().await?;
         let metadata = serde_json::from_slice::<TableMetadata>(&metadata_content)?;
 
-        let table = Table::builder()
+        Table::builder()
             .file_io(self.file_io())
             .metadata_location(metadata_location)
             .metadata(metadata)
@@ -415,9 +415,7 @@ impl Catalog for HmsCatalog {
                 NamespaceIdent::new(db_name),
                 table.name.clone(),
             ))
-            .build();
-
-        Ok(table)
+            .build()
     }
 
     /// Asynchronously drops a table from the database.
