@@ -194,7 +194,9 @@ impl Catalog for MemoryCatalog {
             }
         };
 
-        let metadata = TableMetadataBuilder::from_table_creation(table_creation)?.build()?;
+        let metadata = TableMetadataBuilder::from_table_creation(table_creation)?
+            .build()?
+            .metadata;
         let metadata_location = format!(
             "{}/metadata/{}-{}.metadata.json",
             &location,
@@ -209,14 +211,12 @@ impl Catalog for MemoryCatalog {
 
         root_namespace_state.insert_new_table(&table_ident, metadata_location.clone())?;
 
-        let table = Table::builder()
+        Table::builder()
             .file_io(self.file_io.clone())
             .metadata_location(metadata_location)
             .metadata(metadata)
             .identifier(table_ident)
-            .build();
-
-        Ok(table)
+            .build()
     }
 
     /// Load table from the catalog.
@@ -227,14 +227,13 @@ impl Catalog for MemoryCatalog {
         let input_file = self.file_io.new_input(metadata_location)?;
         let metadata_content = input_file.read().await?;
         let metadata = serde_json::from_slice::<TableMetadata>(&metadata_content)?;
-        let table = Table::builder()
+
+        Table::builder()
             .file_io(self.file_io.clone())
             .metadata_location(metadata_location.clone())
             .metadata(metadata)
             .identifier(table_ident.clone())
-            .build();
-
-        Ok(table)
+            .build()
     }
 
     /// Drop a table from the catalog.
@@ -358,7 +357,7 @@ mod tests {
 
         assert_eq!(metadata.current_schema().as_ref(), expected_schema);
 
-        let expected_partition_spec = PartitionSpec::builder(expected_schema)
+        let expected_partition_spec = PartitionSpec::builder((*expected_schema).clone())
             .with_spec_id(0)
             .build()
             .unwrap();
@@ -374,7 +373,7 @@ mod tests {
         let expected_sorted_order = SortOrder::builder()
             .with_order_id(0)
             .with_fields(vec![])
-            .build(expected_schema.clone())
+            .build(expected_schema)
             .unwrap();
 
         assert_eq!(
