@@ -50,7 +50,7 @@ const PATH_V1: &str = "v1";
 #[derive(Clone, Debug, TypedBuilder)]
 pub struct RestCatalogConfig {
     uri: String,
-    #[builder(default, setter(strip_option))]
+    #[builder(default, setter(strip_option(fallback = warehouse_opt)))]
     warehouse: Option<String>,
 
     #[builder(default)]
@@ -73,12 +73,6 @@ impl RestCatalogConfig {
     pub(crate) fn get_token_endpoint(&self) -> String {
         if let Some(oauth2_uri) = self.props.get("oauth2-server-uri") {
             oauth2_uri.to_string()
-        } else if let Some(auth_url) = self.props.get("rest.authorization-url") {
-            log::warn!(
-                "'rest.authorization-url' is deprecated and will be removed in version 0.4.0. \
-                 Please use 'oauth2-server-uri' instead."
-            );
-            auth_url.to_string()
         } else {
             [&self.uri, PATH_V1, "oauth", "tokens"].join("/")
         }
@@ -256,9 +250,10 @@ impl RestCatalog {
     async fn context(&self) -> Result<&RestContext> {
         self.ctx
             .get_or_try_init(|| async {
-                let catalog_config = RestCatalog::load_config(&self.user_config).await?;
+                let client = HttpClient::new(&self.user_config)?;
+                let catalog_config = RestCatalog::load_config(&client, &self.user_config).await?;
                 let config = self.user_config.clone().merge_with_config(catalog_config);
-                let client = HttpClient::new(&config)?;
+                let client = client.update_with(&config)?;
 
                 Ok(RestContext { config, client })
             })
@@ -268,9 +263,10 @@ impl RestCatalog {
     /// Load the runtime config from the server by user_config.
     ///
     /// It's required for a rest catalog to update it's config after creation.
-    async fn load_config(user_config: &RestCatalogConfig) -> Result<CatalogConfig> {
-        let client = HttpClient::new(user_config)?;
-
+    async fn load_config(
+        client: &HttpClient,
+        user_config: &RestCatalogConfig,
+    ) -> Result<CatalogConfig> {
         let mut request = client.request(Method::GET, user_config.config_endpoint());
 
         if let Some(warehouse_location) = &user_config.warehouse {
@@ -280,6 +276,7 @@ impl RestCatalog {
         let config = client
             .query::<CatalogConfig, ErrorResponse, OK>(request.build()?)
             .await?;
+
         Ok(config)
     }
 
@@ -474,6 +471,11 @@ impl Catalog for RestCatalog {
     }
 
     /// Create a new table inside the namespace.
+    ///
+    /// In the resulting table, if there are any config properties that
+    /// are present in both the response from the REST server and the
+    /// config provided when creating this `RestCatalog` instance then
+    /// the value provided locally to the `RestCatalog` will take precedence.
     async fn create_table(
         &self,
         namespace: &NamespaceIdent,
@@ -512,11 +514,18 @@ impl Catalog for RestCatalog {
             .query::<LoadTableResponse, ErrorResponse, OK>(request)
             .await?;
 
+        let config = resp
+            .config
+            .unwrap_or_default()
+            .into_iter()
+            .chain(self.user_config.props.clone().into_iter())
+            .collect();
+
         let file_io = self
-            .load_file_io(resp.metadata_location.as_deref(), resp.config)
+            .load_file_io(resp.metadata_location.as_deref(), Some(config))
             .await?;
 
-        let table = Table::builder()
+        Table::builder()
             .identifier(table_ident)
             .file_io(file_io)
             .metadata(resp.metadata)
@@ -526,12 +535,15 @@ impl Catalog for RestCatalog {
                     "Metadata location missing in create table response!",
                 )
             })?)
-            .build();
-
-        Ok(table)
+            .build()
     }
 
     /// Load table from the catalog.
+    ///
+    /// If there are any config properties that are present in
+    /// both the response from the REST server and the config provided
+    /// when creating this `RestCatalog` instance then the value
+    /// provided locally to the `RestCatalog` will take precedence.
     async fn load_table(&self, table: &TableIdent) -> Result<Table> {
         let request = self
             .context()
@@ -550,8 +562,15 @@ impl Catalog for RestCatalog {
             .query::<LoadTableResponse, ErrorResponse, OK>(request)
             .await?;
 
+        let config = resp
+            .config
+            .unwrap_or_default()
+            .into_iter()
+            .chain(self.user_config.props.clone().into_iter())
+            .collect();
+
         let file_io = self
-            .load_file_io(resp.metadata_location.as_deref(), resp.config)
+            .load_file_io(resp.metadata_location.as_deref(), Some(config))
             .await?;
 
         let table_builder = Table::builder()
@@ -560,9 +579,9 @@ impl Catalog for RestCatalog {
             .metadata(resp.metadata);
 
         if let Some(metadata_location) = resp.metadata_location {
-            Ok(table_builder.metadata_location(metadata_location).build())
+            table_builder.metadata_location(metadata_location).build()
         } else {
-            Ok(table_builder.build())
+            table_builder.build()
         }
     }
 
@@ -661,12 +680,12 @@ impl Catalog for RestCatalog {
         let file_io = self
             .load_file_io(Some(&resp.metadata_location), None)
             .await?;
-        Ok(Table::builder()
+        Table::builder()
             .identifier(commit.identifier().clone())
             .file_io(file_io)
             .metadata(resp.metadata)
             .metadata_location(resp.metadata_location)
-            .build())
+            .build()
     }
 }
 
@@ -755,7 +774,7 @@ mod tests {
                 "expires_in": 86400
                 }"#,
             )
-            .expect(2)
+            .expect(1)
             .create_async()
             .await
     }
@@ -809,7 +828,7 @@ mod tests {
                 "expires_in": 86400
                 }"#,
             )
-            .expect(2)
+            .expect(1)
             .create_async()
             .await;
 
@@ -899,36 +918,6 @@ mod tests {
         assert_eq!(headers, expected_headers);
     }
 
-    #[tokio::test]
-    async fn test_oauth_with_deprecated_auth_url() {
-        let mut server = Server::new_async().await;
-        let config_mock = create_config_mock(&mut server).await;
-
-        let mut auth_server = Server::new_async().await;
-        let auth_server_path = "/some/path";
-        let oauth_mock = create_oauth_mock_with_path(&mut auth_server, auth_server_path).await;
-
-        let mut props = HashMap::new();
-        props.insert("credential".to_string(), "client1:secret1".to_string());
-        props.insert(
-            "rest.authorization-url".to_string(),
-            format!("{}{}", auth_server.url(), auth_server_path).to_string(),
-        );
-
-        let catalog = RestCatalog::new(
-            RestCatalogConfig::builder()
-                .uri(server.url())
-                .props(props)
-                .build(),
-        );
-
-        let token = catalog.context().await.unwrap().client.token().await;
-
-        oauth_mock.assert_async().await;
-        config_mock.assert_async().await;
-        assert_eq!(token, Some("ey000000000000".to_string()));
-    }
-
     #[tokio::test]
     async fn test_oauth_with_oauth2_server_uri() {
         let mut server = Server::new_async().await;
@@ -1354,7 +1343,7 @@ mod tests {
             .with_schema_id(0)
             .with_summary(Summary {
                 operation: Operation::Append,
-                other: HashMap::from_iter([
+                additional_properties: HashMap::from_iter([
                     ("spark.app.id", "local-1646787004168"),
                     ("added-data-files", "1"),
                     ("added-records", "1"),
@@ -1661,6 +1650,7 @@ mod tests {
                 .identifier(TableIdent::from_strs(["ns1", "test1"]).unwrap())
                 .file_io(FileIO::from_path("/tmp").unwrap().build().unwrap())
                 .build()
+                .unwrap()
         };
 
         let table = Transaction::new(&table1)
@@ -1785,6 +1775,7 @@ mod tests {
                 .identifier(TableIdent::from_strs(["ns1", "test1"]).unwrap())
                 .file_io(FileIO::from_path("/tmp").unwrap().build().unwrap())
                 .build()
+                .unwrap()
         };
 
         let table_result = Transaction::new(&table1)
