@@ -20,4 +20,7 @@
 mod schema;
 pub use schema::*;
 mod reader;
+pub(crate) mod record_batch_projector;
+pub(crate) mod record_batch_transformer;
+
 pub use reader::*;
