@@ -264,9 +264,7 @@ mod tests {
     };
     use crate::Result;
 
-    fn create_schema_and_partition_spec(
-        r#type: PrimitiveType,
-    ) -> Result<(SchemaRef, PartitionSpecRef)> {
+    fn create_partition_spec(r#type: PrimitiveType) -> Result<(PartitionSpecRef, SchemaRef)> {
         let schema = Schema::builder()
             .with_fields(vec![Arc::new(NestedField::optional(
                 1,
@@ -275,28 +273,30 @@ mod tests {
             ))])
             .build()?;
 
-        let spec = PartitionSpec::builder(&schema)
+        let spec = PartitionSpec::builder(schema.clone())
             .with_spec_id(1)
-            .add_unbound_fields(vec![UnboundPartitionField::builder()
-                .source_id(1)
-                .name("a".to_string())
-                .partition_id(1)
-                .transform(Transform::Identity)
-                .build()])
+            .add_unbound_field(
+                UnboundPartitionField::builder()
+                    .source_id(1)
+                    .name("a".to_string())
+                    .field_id(1)
+                    .transform(Transform::Identity)
+                    .build(),
+            )
             .unwrap()
             .build()
             .unwrap();
 
-        Ok((Arc::new(schema), Arc::new(spec)))
+        Ok((Arc::new(spec), schema.into()))
     }
 
     fn create_partition_filter(
-        schema: &Schema,
         partition_spec: PartitionSpecRef,
+        schema: &Schema,
         predicate: &BoundPredicate,
         case_sensitive: bool,
     ) -> Result<BoundPredicate> {
-        let partition_type = partition_spec.partition_type(schema)?;
+        let partition_type = partition_spec.partition_type(schema).unwrap();
         let partition_fields = partition_type.fields().to_owned();
 
         let partition_schema = Schema::builder()
@@ -304,7 +304,7 @@ mod tests {
             .with_fields(partition_fields)
             .build()?;
 
-        let mut inclusive_projection = InclusiveProjection::new(partition_spec);
+        let mut inclusive_projection = InclusiveProjection::new((*partition_spec).clone().into());
 
         let partition_filter = inclusive_projection
             .project(predicate)?
@@ -315,13 +315,13 @@ mod tests {
     }
 
     fn create_expression_evaluator(
-        schema: &Schema,
         partition_spec: PartitionSpecRef,
+        schema: &Schema,
         predicate: &BoundPredicate,
         case_sensitive: bool,
     ) -> Result<ExpressionEvaluator> {
         let partition_filter =
-            create_partition_filter(schema, partition_spec, predicate, case_sensitive)?;
+            create_partition_filter(partition_spec, schema, predicate, case_sensitive)?;
 
         Ok(ExpressionEvaluator::new(partition_filter))
     }
@@ -342,7 +342,7 @@ mod tests {
             nan_value_counts: HashMap::new(),
             lower_bounds: HashMap::new(),
             upper_bounds: HashMap::new(),
-            key_metadata: vec![],
+            key_metadata: None,
             split_offsets: vec![],
             equality_ids: vec![],
             sort_order_id: None,
@@ -365,7 +365,7 @@ mod tests {
             nan_value_counts: HashMap::new(),
             lower_bounds: HashMap::new(),
             upper_bounds: HashMap::new(),
-            key_metadata: vec![],
+            key_metadata: None,
             split_offsets: vec![],
             equality_ids: vec![],
             sort_order_id: None,
@@ -375,7 +375,7 @@ mod tests {
     #[test]
     fn test_expr_or() -> Result<()> {
         let case_sensitive = true;
-        let (schema, partition_spec) = create_schema_and_partition_spec(PrimitiveType::Float)?;
+        let (partition_spec, schema) = create_partition_spec(PrimitiveType::Float)?;
 
         let predicate = Predicate::Binary(BinaryExpression::new(
             PredicateOperator::LessThan,
@@ -390,7 +390,7 @@ mod tests {
         .bind(schema.clone(), case_sensitive)?;
 
         let expression_evaluator =
-            create_expression_evaluator(&schema, partition_spec, &predicate, case_sensitive)?;
+            create_expression_evaluator(partition_spec, &schema, &predicate, case_sensitive)?;
 
         let data_file = create_data_file_float();
 
@@ -404,7 +404,7 @@ mod tests {
     #[test]
     fn test_expr_and() -> Result<()> {
         let case_sensitive = true;
-        let (schema, partition_spec) = create_schema_and_partition_spec(PrimitiveType::Float)?;
+        let (partition_spec, schema) = create_partition_spec(PrimitiveType::Float)?;
 
         let predicate = Predicate::Binary(BinaryExpression::new(
             PredicateOperator::LessThan,
@@ -419,7 +419,7 @@ mod tests {
         .bind(schema.clone(), case_sensitive)?;
 
         let expression_evaluator =
-            create_expression_evaluator(&schema, partition_spec, &predicate, case_sensitive)?;
+            create_expression_evaluator(partition_spec, &schema, &predicate, case_sensitive)?;
 
         let data_file = create_data_file_float();
 
@@ -433,7 +433,7 @@ mod tests {
     #[test]
     fn test_expr_not_in() -> Result<()> {
         let case_sensitive = true;
-        let (schema, partition_spec) = create_schema_and_partition_spec(PrimitiveType::Float)?;
+        let (partition_spec, schema) = create_partition_spec(PrimitiveType::Float)?;
 
         let predicate = Predicate::Set(SetExpression::new(
             PredicateOperator::NotIn,
@@ -443,7 +443,7 @@ mod tests {
         .bind(schema.clone(), case_sensitive)?;
 
         let expression_evaluator =
-            create_expression_evaluator(&schema, partition_spec, &predicate, case_sensitive)?;
+            create_expression_evaluator(partition_spec, &schema, &predicate, case_sensitive)?;
 
         let data_file = create_data_file_float();
 
@@ -457,7 +457,7 @@ mod tests {
     #[test]
     fn test_expr_in() -> Result<()> {
         let case_sensitive = true;
-        let (schema, partition_spec) = create_schema_and_partition_spec(PrimitiveType::Float)?;
+        let (partition_spec, schema) = create_partition_spec(PrimitiveType::Float)?;
 
         let predicate = Predicate::Set(SetExpression::new(
             PredicateOperator::In,
@@ -467,7 +467,7 @@ mod tests {
         .bind(schema.clone(), case_sensitive)?;
 
         let expression_evaluator =
-            create_expression_evaluator(&schema, partition_spec, &predicate, case_sensitive)?;
+            create_expression_evaluator(partition_spec, &schema, &predicate, case_sensitive)?;
 
         let data_file = create_data_file_float();
 
@@ -481,7 +481,7 @@ mod tests {
     #[test]
     fn test_expr_not_starts_with() -> Result<()> {
         let case_sensitive = true;
-        let (schema, partition_spec) = create_schema_and_partition_spec(PrimitiveType::String)?;
+        let (partition_spec, schema) = create_partition_spec(PrimitiveType::String)?;
 
         let predicate = Predicate::Binary(BinaryExpression::new(
             PredicateOperator::NotStartsWith,
@@ -491,7 +491,7 @@ mod tests {
         .bind(schema.clone(), case_sensitive)?;
 
         let expression_evaluator =
-            create_expression_evaluator(&schema, partition_spec, &predicate, case_sensitive)?;
+            create_expression_evaluator(partition_spec, &schema, &predicate, case_sensitive)?;
 
         let data_file = create_data_file_string();
 
@@ -505,7 +505,7 @@ mod tests {
     #[test]
     fn test_expr_starts_with() -> Result<()> {
         let case_sensitive = true;
-        let (schema, partition_spec) = create_schema_and_partition_spec(PrimitiveType::String)?;
+        let (partition_spec, schema) = create_partition_spec(PrimitiveType::String)?;
 
         let predicate = Predicate::Binary(BinaryExpression::new(
             PredicateOperator::StartsWith,
@@ -515,7 +515,7 @@ mod tests {
         .bind(schema.clone(), case_sensitive)?;
 
         let expression_evaluator =
-            create_expression_evaluator(&schema, partition_spec, &predicate, case_sensitive)?;
+            create_expression_evaluator(partition_spec, &schema, &predicate, case_sensitive)?;
 
         let data_file = create_data_file_string();
 
@@ -529,7 +529,7 @@ mod tests {
     #[test]
     fn test_expr_not_eq() -> Result<()> {
         let case_sensitive = true;
-        let (schema, partition_spec) = create_schema_and_partition_spec(PrimitiveType::Float)?;
+        let (partition_spec, schema) = create_partition_spec(PrimitiveType::Float)?;
 
         let predicate = Predicate::Binary(BinaryExpression::new(
             PredicateOperator::NotEq,
@@ -539,7 +539,7 @@ mod tests {
         .bind(schema.clone(), case_sensitive)?;
 
         let expression_evaluator =
-            create_expression_evaluator(&schema, partition_spec, &predicate, case_sensitive)?;
+            create_expression_evaluator(partition_spec, &schema, &predicate, case_sensitive)?;
 
         let data_file = create_data_file_float();
 
@@ -553,7 +553,7 @@ mod tests {
     #[test]
     fn test_expr_eq() -> Result<()> {
         let case_sensitive = true;
-        let (schema, partition_spec) = create_schema_and_partition_spec(PrimitiveType::Float)?;
+        let (partition_spec, schema) = create_partition_spec(PrimitiveType::Float)?;
 
         let predicate = Predicate::Binary(BinaryExpression::new(
             PredicateOperator::Eq,
@@ -563,7 +563,7 @@ mod tests {
         .bind(schema.clone(), case_sensitive)?;
 
         let expression_evaluator =
-            create_expression_evaluator(&schema, partition_spec, &predicate, case_sensitive)?;
+            create_expression_evaluator(partition_spec, &schema, &predicate, case_sensitive)?;
 
         let data_file = create_data_file_float();
 
@@ -577,7 +577,7 @@ mod tests {
     #[test]
     fn test_expr_greater_than_or_eq() -> Result<()> {
         let case_sensitive = true;
-        let (schema, partition_spec) = create_schema_and_partition_spec(PrimitiveType::Float)?;
+        let (partition_spec, schema) = create_partition_spec(PrimitiveType::Float)?;
 
         let predicate = Predicate::Binary(BinaryExpression::new(
             PredicateOperator::GreaterThanOrEq,
@@ -587,7 +587,7 @@ mod tests {
         .bind(schema.clone(), case_sensitive)?;
 
         let expression_evaluator =
-            create_expression_evaluator(&schema, partition_spec, &predicate, case_sensitive)?;
+            create_expression_evaluator(partition_spec, &schema, &predicate, case_sensitive)?;
 
         let data_file = create_data_file_float();
 
@@ -601,7 +601,7 @@ mod tests {
     #[test]
     fn test_expr_greater_than() -> Result<()> {
         let case_sensitive = true;
-        let (schema, partition_spec) = create_schema_and_partition_spec(PrimitiveType::Float)?;
+        let (partition_spec, schema) = create_partition_spec(PrimitiveType::Float)?;
 
         let predicate = Predicate::Binary(BinaryExpression::new(
             PredicateOperator::GreaterThan,
@@ -611,7 +611,7 @@ mod tests {
         .bind(schema.clone(), case_sensitive)?;
 
         let expression_evaluator =
-            create_expression_evaluator(&schema, partition_spec, &predicate, case_sensitive)?;
+            create_expression_evaluator(partition_spec, &schema, &predicate, case_sensitive)?;
 
         let data_file = create_data_file_float();
 
@@ -625,7 +625,7 @@ mod tests {
     #[test]
     fn test_expr_less_than_or_eq() -> Result<()> {
         let case_sensitive = true;
-        let (schema, partition_spec) = create_schema_and_partition_spec(PrimitiveType::Float)?;
+        let (partition_spec, schema) = create_partition_spec(PrimitiveType::Float)?;
 
         let predicate = Predicate::Binary(BinaryExpression::new(
             PredicateOperator::LessThanOrEq,
@@ -635,7 +635,7 @@ mod tests {
         .bind(schema.clone(), case_sensitive)?;
 
         let expression_evaluator =
-            create_expression_evaluator(&schema, partition_spec, &predicate, case_sensitive)?;
+            create_expression_evaluator(partition_spec, &schema, &predicate, case_sensitive)?;
 
         let data_file = create_data_file_float();
 
@@ -649,7 +649,7 @@ mod tests {
     #[test]
     fn test_expr_less_than() -> Result<()> {
         let case_sensitive = true;
-        let (schema, partition_spec) = create_schema_and_partition_spec(PrimitiveType::Float)?;
+        let (partition_spec, schema) = create_partition_spec(PrimitiveType::Float)?;
 
         let predicate = Predicate::Binary(BinaryExpression::new(
             PredicateOperator::LessThan,
@@ -659,7 +659,7 @@ mod tests {
         .bind(schema.clone(), case_sensitive)?;
 
         let expression_evaluator =
-            create_expression_evaluator(&schema, partition_spec, &predicate, case_sensitive)?;
+            create_expression_evaluator(partition_spec, &schema, &predicate, case_sensitive)?;
 
         let data_file = create_data_file_float();
 
@@ -673,7 +673,7 @@ mod tests {
     #[test]
     fn test_expr_is_not_nan() -> Result<()> {
         let case_sensitive = true;
-        let (schema, partition_spec) = create_schema_and_partition_spec(PrimitiveType::Float)?;
+        let (partition_spec, schema) = create_partition_spec(PrimitiveType::Float)?;
         let predicate = Predicate::Unary(UnaryExpression::new(
             PredicateOperator::NotNan,
             Reference::new("a"),
@@ -681,7 +681,7 @@ mod tests {
         .bind(schema.clone(), case_sensitive)?;
 
         let expression_evaluator =
-            create_expression_evaluator(&schema, partition_spec, &predicate, case_sensitive)?;
+            create_expression_evaluator(partition_spec, &schema, &predicate, case_sensitive)?;
 
         let data_file = create_data_file_float();
 
@@ -695,7 +695,7 @@ mod tests {
     #[test]
     fn test_expr_is_nan() -> Result<()> {
         let case_sensitive = true;
-        let (schema, partition_spec) = create_schema_and_partition_spec(PrimitiveType::Float)?;
+        let (partition_spec, schema) = create_partition_spec(PrimitiveType::Float)?;
         let predicate = Predicate::Unary(UnaryExpression::new(
             PredicateOperator::IsNan,
             Reference::new("a"),
@@ -703,7 +703,7 @@ mod tests {
         .bind(schema.clone(), case_sensitive)?;
 
         let expression_evaluator =
-            create_expression_evaluator(&schema, partition_spec, &predicate, case_sensitive)?;
+            create_expression_evaluator(partition_spec, &schema, &predicate, case_sensitive)?;
 
         let data_file = create_data_file_float();
 
@@ -717,7 +717,7 @@ mod tests {
     #[test]
     fn test_expr_is_not_null() -> Result<()> {
         let case_sensitive = true;
-        let (schema, partition_spec) = create_schema_and_partition_spec(PrimitiveType::Float)?;
+        let (partition_spec, schema) = create_partition_spec(PrimitiveType::Float)?;
         let predicate = Predicate::Unary(UnaryExpression::new(
             PredicateOperator::NotNull,
             Reference::new("a"),
@@ -725,7 +725,7 @@ mod tests {
         .bind(schema.clone(), case_sensitive)?;
 
         let expression_evaluator =
-            create_expression_evaluator(&schema, partition_spec, &predicate, case_sensitive)?;
+            create_expression_evaluator(partition_spec, &schema, &predicate, case_sensitive)?;
 
         let data_file = create_data_file_float();
 
@@ -739,7 +739,7 @@ mod tests {
     #[test]
     fn test_expr_is_null() -> Result<()> {
         let case_sensitive = true;
-        let (schema, partition_spec) = create_schema_and_partition_spec(PrimitiveType::Float)?;
+        let (partition_spec, schema) = create_partition_spec(PrimitiveType::Float)?;
         let predicate = Predicate::Unary(UnaryExpression::new(
             PredicateOperator::IsNull,
             Reference::new("a"),
@@ -747,7 +747,7 @@ mod tests {
         .bind(schema.clone(), case_sensitive)?;
 
         let expression_evaluator =
-            create_expression_evaluator(&schema, partition_spec, &predicate, case_sensitive)?;
+            create_expression_evaluator(partition_spec, &schema, &predicate, case_sensitive)?;
 
         let data_file = create_data_file_float();
 
@@ -761,11 +761,11 @@ mod tests {
     #[test]
     fn test_expr_always_false() -> Result<()> {
         let case_sensitive = true;
-        let (schema, partition_spec) = create_schema_and_partition_spec(PrimitiveType::Float)?;
+        let (partition_spec, schema) = create_partition_spec(PrimitiveType::Float)?;
         let predicate = Predicate::AlwaysFalse.bind(schema.clone(), case_sensitive)?;
 
         let expression_evaluator =
-            create_expression_evaluator(&schema, partition_spec, &predicate, case_sensitive)?;
+            create_expression_evaluator(partition_spec, &schema, &predicate, case_sensitive)?;
 
         let data_file = create_data_file_float();
 
@@ -779,11 +779,11 @@ mod tests {
     #[test]
     fn test_expr_always_true() -> Result<()> {
         let case_sensitive = true;
-        let (schema, partition_spec) = create_schema_and_partition_spec(PrimitiveType::Float)?;
+        let (partition_spec, schema) = create_partition_spec(PrimitiveType::Float)?;
         let predicate = Predicate::AlwaysTrue.bind(schema.clone(), case_sensitive)?;
 
         let expression_evaluator =
-            create_expression_evaluator(&schema, partition_spec, &predicate, case_sensitive)?;
+            create_expression_evaluator(partition_spec, &schema, &predicate, case_sensitive)?;
 
         let data_file = create_data_file_float();
 
