@@ -20,11 +20,14 @@
 mod datatypes;
 mod manifest;
 mod manifest_list;
+mod name_mapping;
 mod partition;
 mod schema;
 mod snapshot;
 mod sort;
+mod statistic_file;
 mod table_metadata;
+mod table_metadata_builder;
 mod transform;
 mod values;
 mod view_metadata;
@@ -37,6 +40,7 @@ pub use partition::*;
 pub use schema::*;
 pub use snapshot::*;
 pub use sort::*;
+pub use statistic_file::*;
 pub use table_metadata::*;
 pub use transform::*;
 pub use values::*;
