@@ -24,14 +24,12 @@ use serde::{Deserialize, Serialize};
 use typed_builder::TypedBuilder;
 
 use super::transform::Transform;
-use super::{NestedField, Schema, StructType};
+use super::{NestedField, Schema, SchemaRef, StructType};
 use crate::{Error, ErrorKind, Result};
 
 pub(crate) const UNPARTITIONED_LAST_ASSIGNED_ID: i32 = 999;
 pub(crate) const DEFAULT_PARTITION_SPEC_ID: i32 = 0;
 
-/// Reference to [`PartitionSpec`].
-pub type PartitionSpecRef = Arc<PartitionSpec>;
 /// Partition fields capture the transform from table data to partition values.
 #[derive(Debug, Serialize, Deserialize, PartialEq, Eq, Clone, TypedBuilder)]
 #[serde(rename_all = "kebab-case")]
@@ -54,73 +52,106 @@ impl PartitionField {
     }
 }
 
-///  Partition spec that defines how to produce a tuple of partition values from a record.
-#[derive(Debug, Serialize, Deserialize, PartialEq, Eq, Clone, Default)]
+/// Reference to [`PartitionSpec`].
+pub type PartitionSpecRef = Arc<PartitionSpec>;
+/// Partition spec that defines how to produce a tuple of partition values from a record.
+///
+/// A [`PartitionSpec`] is originally obtained by binding an [`UnboundPartitionSpec`] to a schema and is
+/// only guaranteed to be valid for that schema. The main difference between [`PartitionSpec`] and
+/// [`UnboundPartitionSpec`] is that the former has field ids assigned,
+/// while field ids are optional for [`UnboundPartitionSpec`].
+#[derive(Debug, Serialize, Deserialize, PartialEq, Eq, Clone)]
 #[serde(rename_all = "kebab-case")]
 pub struct PartitionSpec {
     /// Identifier for PartitionSpec
-    pub(crate) spec_id: i32,
+    spec_id: i32,
     /// Details of the partition spec
-    pub(crate) fields: Vec<PartitionField>,
+    fields: Vec<PartitionField>,
 }
 
 impl PartitionSpec {
-    /// Create partition spec builer
-    pub fn builder(schema: &Schema) -> PartitionSpecBuilder {
+    /// Create a new partition spec builder with the given schema.
+    pub fn builder(schema: impl Into<SchemaRef>) -> PartitionSpecBuilder {
         PartitionSpecBuilder::new(schema)
     }
 
-    /// Spec id of the partition spec
-    pub fn spec_id(&self) -> i32 {
-        self.spec_id
-    }
-
     /// Fields of the partition spec
     pub fn fields(&self) -> &[PartitionField] {
         &self.fields
     }
 
+    /// Spec id of the partition spec
+    pub fn spec_id(&self) -> i32 {
+        self.spec_id
+    }
+
+    /// Get a new unpartitioned partition spec
+    pub fn unpartition_spec() -> Self {
+        Self {
+            spec_id: DEFAULT_PARTITION_SPEC_ID,
+            fields: vec![],
+        }
+    }
+
     /// Returns if the partition spec is unpartitioned.
     ///
     /// A [`PartitionSpec`] is unpartitioned if it has no fields or all fields are [`Transform::Void`] transform.
     pub fn is_unpartitioned(&self) -> bool {
-        self.fields.is_empty()
-            || self
-                .fields
-                .iter()
-                .all(|f| matches!(f.transform, Transform::Void))
+        self.fields.is_empty() || self.fields.iter().all(|f| f.transform == Transform::Void)
     }
 
     /// Returns the partition type of this partition spec.
     pub fn partition_type(&self, schema: &Schema) -> Result<StructType> {
-        let mut fields = Vec::with_capacity(self.fields.len());
-        for partition_field in &self.fields {
-            let field = schema
-                .field_by_id(partition_field.source_id)
-                .ok_or_else(|| {
-                    Error::new(
-                        ErrorKind::DataInvalid,
-                        format!(
-                            "No column with source column id {} in schema {:?}",
-                            partition_field.source_id, schema
-                        ),
-                    )
-                })?;
-            let res_type = partition_field.transform.result_type(&field.field_type)?;
-            let field =
-                NestedField::optional(partition_field.field_id, &partition_field.name, res_type)
-                    .into();
-            fields.push(field);
-        }
-        Ok(StructType::new(fields))
+        PartitionSpecBuilder::partition_type(&self.fields, schema)
     }
 
-    /// Turn this partition spec into an unbound partition spec.
-    ///
-    /// The `field_id` is retained as `partition_id` in the unbound partition spec.
-    pub fn to_unbound(self) -> UnboundPartitionSpec {
+    /// Convert to unbound partition spec
+    pub fn into_unbound(self) -> UnboundPartitionSpec {
         self.into()
     }
+
+    /// Change the spec id of the partition spec
+    pub fn with_spec_id(self, spec_id: i32) -> Self {
+        Self { spec_id, ..self }
+    }
+
+    /// Check if this partition spec has sequential partition ids.
+    /// Sequential ids start from 1000 and increment by 1 for each field.
+    /// This is required for spec version 1
+    pub fn has_sequential_ids(&self) -> bool {
+        has_sequential_ids(self.fields.iter().map(|f| f.field_id))
+    }
+
+    /// Get the highest field id in the partition spec.
+    pub fn highest_field_id(&self) -> Option<i32> {
+        self.fields.iter().map(|f| f.field_id).max()
+    }
+
+    /// Check if this partition spec is compatible with another partition spec.
+    ///
+    /// Returns true if the partition spec is equal to the other spec with partition field ids ignored and
+    /// spec_id ignored. The following must be identical:
+    /// * The number of fields
+    /// * Field order
+    /// * Field names
+    /// * Source column ids
+    /// * Transforms
+    pub fn is_compatible_with(&self, other: &PartitionSpec) -> bool {
+        if self.fields.len() != other.fields.len() {
+            return false;
+        }
+
+        for (this_field, other_field) in self.fields.iter().zip(other.fields.iter()) {
+            if this_field.source_id != other_field.source_id
+                || this_field.name != other_field.name
+                || this_field.transform != other_field.transform
+            {
+                return false;
+            }
+        }
+
+        true
+    }
 }
 
 /// Reference to [`UnboundPartitionSpec`].
@@ -133,8 +164,8 @@ pub struct UnboundPartitionField {
     pub source_id: i32,
     /// A partition field id that is used to identify a partition field and is unique within a partition spec.
     /// In v2 table metadata, it is unique across all partition specs.
-    #[builder(default, setter(strip_option))]
-    pub partition_id: Option<i32>,
+    #[builder(default, setter(strip_option(fallback = field_id_opt)))]
+    pub field_id: Option<i32>,
     /// A partition name.
     pub name: String,
     /// A transform that is applied to the source column to produce a partition value.
@@ -142,6 +173,8 @@ pub struct UnboundPartitionField {
 }
 
 /// Unbound partition spec can be built without a schema and later bound to a schema.
+/// They are used to transport schema information as part of the REST specification.
+/// The main difference to [`PartitionSpec`] is that the field ids are optional.
 #[derive(Debug, Serialize, Deserialize, PartialEq, Eq, Clone, Default)]
 #[serde(rename_all = "kebab-case")]
 pub struct UnboundPartitionSpec {
@@ -152,13 +185,13 @@ pub struct UnboundPartitionSpec {
 }
 
 impl UnboundPartitionSpec {
-    /// Create unbound partition spec builer
+    /// Create unbound partition spec builder
     pub fn builder() -> UnboundPartitionSpecBuilder {
         UnboundPartitionSpecBuilder::default()
     }
 
     /// Bind this unbound partition spec to a schema.
-    pub fn bind(self, schema: &Schema) -> Result<PartitionSpec> {
+    pub fn bind(self, schema: impl Into<SchemaRef>) -> Result<PartitionSpec> {
         PartitionSpecBuilder::new_from_unbound(self, schema)?.build()
     }
 
@@ -171,13 +204,36 @@ impl UnboundPartitionSpec {
     pub fn fields(&self) -> &[UnboundPartitionField] {
         &self.fields
     }
+
+    /// Change the spec id of the partition spec
+    pub fn with_spec_id(self, spec_id: i32) -> Self {
+        Self {
+            spec_id: Some(spec_id),
+            ..self
+        }
+    }
+}
+
+fn has_sequential_ids(field_ids: impl Iterator<Item = i32>) -> bool {
+    for (index, field_id) in field_ids.enumerate() {
+        let expected_id = (UNPARTITIONED_LAST_ASSIGNED_ID as i64)
+            .checked_add(1)
+            .and_then(|id| id.checked_add(index as i64))
+            .unwrap_or(i64::MAX);
+
+        if field_id as i64 != expected_id {
+            return false;
+        }
+    }
+
+    true
 }
 
 impl From<PartitionField> for UnboundPartitionField {
     fn from(field: PartitionField) -> Self {
         UnboundPartitionField {
             source_id: field.source_id,
-            partition_id: Some(field.field_id),
+            field_id: Some(field.field_id),
             name: field.name,
             transform: field.transform,
         }
@@ -224,7 +280,7 @@ impl UnboundPartitionSpecBuilder {
     ) -> Result<Self> {
         let field = UnboundPartitionField {
             source_id,
-            partition_id: None,
+            field_id: None,
             name: target_name.to_string(),
             transform: transformation,
         };
@@ -246,8 +302,8 @@ impl UnboundPartitionSpecBuilder {
     fn add_partition_field_internal(mut self, field: UnboundPartitionField) -> Result<Self> {
         self.check_name_set_and_unique(&field.name)?;
         self.check_for_redundant_partitions(field.source_id, &field.transform)?;
-        if let Some(partition_id) = field.partition_id {
-            self.check_partition_id_unique(partition_id)?;
+        if let Some(partition_field_id) = field.field_id {
+            self.check_partition_id_unique(partition_field_id)?;
         }
         self.fields.push(field);
         Ok(self)
@@ -264,26 +320,29 @@ impl UnboundPartitionSpecBuilder {
 
 /// Create valid partition specs for a given schema.
 #[derive(Debug)]
-pub struct PartitionSpecBuilder<'a> {
+pub struct PartitionSpecBuilder {
     spec_id: Option<i32>,
     last_assigned_field_id: i32,
     fields: Vec<UnboundPartitionField>,
-    schema: &'a Schema,
+    schema: SchemaRef,
 }
 
-impl<'a> PartitionSpecBuilder<'a> {
+impl PartitionSpecBuilder {
     /// Create a new partition spec builder with the given schema.
-    pub fn new(schema: &'a Schema) -> Self {
+    pub fn new(schema: impl Into<SchemaRef>) -> Self {
         Self {
             spec_id: None,
             fields: vec![],
             last_assigned_field_id: UNPARTITIONED_LAST_ASSIGNED_ID,
-            schema,
+            schema: schema.into(),
         }
     }
 
     /// Create a new partition spec builder from an existing unbound partition spec.
-    pub fn new_from_unbound(unbound: UnboundPartitionSpec, schema: &'a Schema) -> Result<Self> {
+    pub fn new_from_unbound(
+        unbound: UnboundPartitionSpec,
+        schema: impl Into<SchemaRef>,
+    ) -> Result<Self> {
         let mut builder =
             Self::new(schema).with_spec_id(unbound.spec_id.unwrap_or(DEFAULT_PARTITION_SPEC_ID));
 
@@ -331,7 +390,7 @@ impl<'a> PartitionSpecBuilder<'a> {
             .id;
         let field = UnboundPartitionField {
             source_id,
-            partition_id: None,
+            field_id: None,
             name: target_name.into(),
             transform,
         };
@@ -341,15 +400,15 @@ impl<'a> PartitionSpecBuilder<'a> {
 
     /// Add a new partition field to the partition spec.
     ///
-    /// If `partition_id` is set, it is used as the field id.
+    /// If partition field id is set, it is used as the field id.
     /// Otherwise, a new `field_id` is assigned.
     pub fn add_unbound_field(mut self, field: UnboundPartitionField) -> Result<Self> {
         self.check_name_set_and_unique(&field.name)?;
         self.check_for_redundant_partitions(field.source_id, &field.transform)?;
-        Self::check_name_does_not_collide_with_schema(&field, self.schema)?;
-        Self::check_transform_compatibility(&field, self.schema)?;
-        if let Some(partition_id) = field.partition_id {
-            self.check_partition_id_unique(partition_id)?;
+        Self::check_name_does_not_collide_with_schema(&field, &self.schema)?;
+        Self::check_transform_compatibility(&field, &self.schema)?;
+        if let Some(partition_field_id) = field.field_id {
+            self.check_partition_id_unique(partition_field_id)?;
         }
 
         // Non-fallible from here
@@ -387,7 +446,7 @@ impl<'a> PartitionSpecBuilder<'a> {
         // we skip it.
         let assigned_ids = fields
             .iter()
-            .filter_map(|f| f.partition_id)
+            .filter_map(|f| f.field_id)
             .collect::<std::collections::HashSet<_>>();
 
         fn _check_add_1(prev: i32) -> Result<i32> {
@@ -401,9 +460,9 @@ impl<'a> PartitionSpecBuilder<'a> {
 
         let mut bound_fields = Vec::with_capacity(fields.len());
         for field in fields.into_iter() {
-            let partition_id = if let Some(partition_id) = field.partition_id {
-                last_assigned_field_id = std::cmp::max(last_assigned_field_id, partition_id);
-                partition_id
+            let partition_field_id = if let Some(partition_field_id) = field.field_id {
+                last_assigned_field_id = std::cmp::max(last_assigned_field_id, partition_field_id);
+                partition_field_id
             } else {
                 last_assigned_field_id = _check_add_1(last_assigned_field_id)?;
                 while assigned_ids.contains(&last_assigned_field_id) {
@@ -414,7 +473,7 @@ impl<'a> PartitionSpecBuilder<'a> {
 
             bound_fields.push(PartitionField {
                 source_id: field.source_id,
-                field_id: partition_id,
+                field_id: partition_field_id,
                 name: field.name,
                 transform: field.transform,
             })
@@ -423,6 +482,32 @@ impl<'a> PartitionSpecBuilder<'a> {
         Ok(bound_fields)
     }
 
+    /// Returns the partition type of this partition spec.
+    fn partition_type(fields: &Vec<PartitionField>, schema: &Schema) -> Result<StructType> {
+        let mut struct_fields = Vec::with_capacity(fields.len());
+        for partition_field in fields {
+            let field = schema
+                .field_by_id(partition_field.source_id)
+                .ok_or_else(|| {
+                    Error::new(
+                        // This should never occur as check_transform_compatibility
+                        // already ensures that the source field exists in the schema
+                        ErrorKind::Unexpected,
+                        format!(
+                            "No column with source column id {} in schema {:?}",
+                            partition_field.source_id, schema
+                        ),
+                    )
+                })?;
+            let res_type = partition_field.transform.result_type(&field.field_type)?;
+            let field =
+                NestedField::optional(partition_field.field_id, &partition_field.name, res_type)
+                    .into();
+            struct_fields.push(field);
+        }
+        Ok(StructType::new(struct_fields))
+    }
+
     /// Ensure that the partition name is unique among columns in the schema.
     /// Duplicate names are allowed if:
     /// 1. The column is sourced from the column with the same name.
@@ -544,11 +629,7 @@ trait CorePartitionSpecValidator {
 
     /// Check field / partition_id unique within the partition spec if set
     fn check_partition_id_unique(&self, field_id: i32) -> Result<()> {
-        if self
-            .fields()
-            .iter()
-            .any(|f| f.partition_id == Some(field_id))
-        {
+        if self.fields().iter().any(|f| f.field_id == Some(field_id)) {
             return Err(Error::new(
                 ErrorKind::DataInvalid,
                 format!(
@@ -564,7 +645,7 @@ trait CorePartitionSpecValidator {
     fn fields(&self) -> &Vec<UnboundPartitionField>;
 }
 
-impl CorePartitionSpecValidator for PartitionSpecBuilder<'_> {
+impl CorePartitionSpecValidator for PartitionSpecBuilder {
     fn fields(&self) -> &Vec<UnboundPartitionField> {
         &self.fields
     }
@@ -579,7 +660,7 @@ impl CorePartitionSpecValidator for UnboundPartitionSpecBuilder {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::spec::Type;
+    use crate::spec::{PrimitiveType, Type};
 
     #[test]
     fn test_partition_spec() {
@@ -637,7 +718,7 @@ mod tests {
             ])
             .build()
             .unwrap();
-        let partition_spec = PartitionSpec::builder(&schema)
+        let partition_spec = PartitionSpec::builder(schema.clone())
             .with_spec_id(1)
             .build()
             .unwrap();
@@ -646,7 +727,7 @@ mod tests {
             "Empty partition spec should be unpartitioned"
         );
 
-        let partition_spec = PartitionSpec::builder(&schema)
+        let partition_spec = PartitionSpec::builder(schema.clone())
             .add_unbound_fields(vec![
                 UnboundPartitionField::builder()
                     .source_id(1)
@@ -668,7 +749,7 @@ mod tests {
             "Partition spec with one non void transform should not be unpartitioned"
         );
 
-        let partition_spec = PartitionSpec::builder(&schema)
+        let partition_spec = PartitionSpec::builder(schema.clone())
             .with_spec_id(1)
             .add_unbound_fields(vec![
                 UnboundPartitionField::builder()
@@ -698,17 +779,17 @@ mod tests {
 		"spec-id": 1,
 		"fields": [ {
 			"source-id": 4,
-			"partition-id": 1000,
+			"field-id": 1000,
 			"name": "ts_day",
 			"transform": "day"
 			}, {
 			"source-id": 1,
-			"partition-id": 1001,
+			"field-id": 1001,
 			"name": "id_bucket",
 			"transform": "bucket[16]"
 			}, {
 			"source-id": 2,
-			"partition-id": 1002,
+			"field-id": 1002,
 			"name": "id_truncate",
 			"transform": "truncate[4]"
 			} ]
@@ -719,17 +800,17 @@ mod tests {
         assert_eq!(Some(1), partition_spec.spec_id);
 
         assert_eq!(4, partition_spec.fields[0].source_id);
-        assert_eq!(Some(1000), partition_spec.fields[0].partition_id);
+        assert_eq!(Some(1000), partition_spec.fields[0].field_id);
         assert_eq!("ts_day", partition_spec.fields[0].name);
         assert_eq!(Transform::Day, partition_spec.fields[0].transform);
 
         assert_eq!(1, partition_spec.fields[1].source_id);
-        assert_eq!(Some(1001), partition_spec.fields[1].partition_id);
+        assert_eq!(Some(1001), partition_spec.fields[1].field_id);
         assert_eq!("id_bucket", partition_spec.fields[1].name);
         assert_eq!(Transform::Bucket(16), partition_spec.fields[1].transform);
 
         assert_eq!(2, partition_spec.fields[2].source_id);
-        assert_eq!(Some(1002), partition_spec.fields[2].partition_id);
+        assert_eq!(Some(1002), partition_spec.fields[2].field_id);
         assert_eq!("id_truncate", partition_spec.fields[2].name);
         assert_eq!(Transform::Truncate(4), partition_spec.fields[2].transform);
 
@@ -746,11 +827,37 @@ mod tests {
         assert_eq!(None, partition_spec.spec_id);
 
         assert_eq!(4, partition_spec.fields[0].source_id);
-        assert_eq!(None, partition_spec.fields[0].partition_id);
+        assert_eq!(None, partition_spec.fields[0].field_id);
         assert_eq!("ts_day", partition_spec.fields[0].name);
         assert_eq!(Transform::Day, partition_spec.fields[0].transform);
     }
 
+    #[test]
+    fn test_new_unpartition() {
+        let schema = Schema::builder()
+            .with_fields(vec![
+                NestedField::required(1, "id", Type::Primitive(crate::spec::PrimitiveType::Int))
+                    .into(),
+                NestedField::required(
+                    2,
+                    "name",
+                    Type::Primitive(crate::spec::PrimitiveType::String),
+                )
+                .into(),
+            ])
+            .build()
+            .unwrap();
+        let partition_spec = PartitionSpec::builder(schema.clone())
+            .with_spec_id(0)
+            .build()
+            .unwrap();
+        let partition_type = partition_spec.partition_type(&schema).unwrap();
+        assert_eq!(0, partition_type.fields().len());
+
+        let unpartition_spec = PartitionSpec::unpartition_spec();
+        assert_eq!(partition_spec, unpartition_spec);
+    }
+
     #[test]
     fn test_partition_type() {
         let spec = r#"
@@ -960,17 +1067,17 @@ mod tests {
             ])
             .build()
             .unwrap();
-        PartitionSpec::builder(&schema)
+        PartitionSpec::builder(schema.clone())
             .add_unbound_field(UnboundPartitionField {
                 source_id: 1,
-                partition_id: Some(1000),
+                field_id: Some(1000),
                 name: "id".to_string(),
                 transform: Transform::Identity,
             })
             .unwrap()
             .add_unbound_field(UnboundPartitionField {
                 source_id: 2,
-                partition_id: Some(1000),
+                field_id: Some(1000),
                 name: "id_bucket".to_string(),
                 transform: Transform::Bucket(16),
             })
@@ -998,20 +1105,20 @@ mod tests {
             ])
             .build()
             .unwrap();
-        let spec = PartitionSpec::builder(&schema)
+        let spec = PartitionSpec::builder(schema.clone())
             .with_spec_id(1)
             .add_unbound_field(UnboundPartitionField {
                 source_id: 1,
                 name: "id".to_string(),
                 transform: Transform::Identity,
-                partition_id: Some(1012),
+                field_id: Some(1012),
             })
             .unwrap()
             .add_unbound_field(UnboundPartitionField {
                 source_id: 2,
                 name: "name_void".to_string(),
                 transform: Transform::Void,
-                partition_id: None,
+                field_id: None,
             })
             .unwrap()
             // Should keep its ID even if its lower
@@ -1019,7 +1126,7 @@ mod tests {
                 source_id: 3,
                 name: "year".to_string(),
                 transform: Transform::Year,
-                partition_id: Some(1),
+                field_id: Some(1),
             })
             .unwrap()
             .build()
@@ -1046,12 +1153,12 @@ mod tests {
             .build()
             .unwrap();
 
-        PartitionSpec::builder(&schema)
+        PartitionSpec::builder(schema.clone())
             .with_spec_id(1)
             .build()
             .unwrap();
 
-        let spec = PartitionSpec::builder(&schema)
+        let spec = PartitionSpec::builder(schema.clone())
             .with_spec_id(1)
             .add_partition_field("id", "id_bucket[16]", Transform::Bucket(16))
             .unwrap()
@@ -1065,8 +1172,17 @@ mod tests {
                 field_id: 1000,
                 name: "id_bucket[16]".to_string(),
                 transform: Transform::Bucket(16),
-            }]
+            }],
         });
+        assert_eq!(
+            spec.partition_type(&schema).unwrap(),
+            StructType::new(vec![NestedField::optional(
+                1000,
+                "id_bucket[16]",
+                Type::Primitive(PrimitiveType::Int)
+            )
+            .into()])
+        )
     }
 
     #[test]
@@ -1081,16 +1197,16 @@ mod tests {
             .build()
             .unwrap();
 
-        PartitionSpec::builder(&schema)
+        PartitionSpec::builder(schema.clone())
             .with_spec_id(1)
             .build()
             .unwrap();
 
-        let err = PartitionSpec::builder(&schema)
+        let err = PartitionSpec::builder(schema)
             .with_spec_id(1)
             .add_unbound_field(UnboundPartitionField {
                 source_id: 1,
-                partition_id: None,
+                field_id: None,
                 name: "id".to_string(),
                 transform: Transform::Bucket(16),
             })
@@ -1114,16 +1230,16 @@ mod tests {
             .build()
             .unwrap();
 
-        PartitionSpec::builder(&schema)
+        PartitionSpec::builder(schema.clone())
             .with_spec_id(1)
             .build()
             .unwrap();
 
-        PartitionSpec::builder(&schema)
+        PartitionSpec::builder(schema.clone())
             .with_spec_id(1)
             .add_unbound_field(UnboundPartitionField {
                 source_id: 1,
-                partition_id: None,
+                field_id: None,
                 name: "id".to_string(),
                 transform: Transform::Identity,
             })
@@ -1132,11 +1248,11 @@ mod tests {
             .unwrap();
 
         // Not OK for different source id
-        PartitionSpec::builder(&schema)
+        PartitionSpec::builder(schema)
             .with_spec_id(1)
             .add_unbound_field(UnboundPartitionField {
                 source_id: 2,
-                partition_id: None,
+                field_id: None,
                 name: "id".to_string(),
                 transform: Transform::Identity,
             })
@@ -1166,18 +1282,18 @@ mod tests {
             .unwrap();
 
         // Valid
-        PartitionSpec::builder(&schema)
+        PartitionSpec::builder(schema.clone())
             .with_spec_id(1)
             .add_unbound_fields(vec![
                 UnboundPartitionField {
                     source_id: 1,
-                    partition_id: None,
+                    field_id: None,
                     name: "id_bucket".to_string(),
                     transform: Transform::Bucket(16),
                 },
                 UnboundPartitionField {
                     source_id: 2,
-                    partition_id: None,
+                    field_id: None,
                     name: "name".to_string(),
                     transform: Transform::Identity,
                 },
@@ -1187,18 +1303,18 @@ mod tests {
             .unwrap();
 
         // Invalid
-        PartitionSpec::builder(&schema)
+        PartitionSpec::builder(schema)
             .with_spec_id(1)
             .add_unbound_fields(vec![
                 UnboundPartitionField {
                     source_id: 1,
-                    partition_id: None,
+                    field_id: None,
                     name: "id_bucket".to_string(),
                     transform: Transform::Bucket(16),
                 },
                 UnboundPartitionField {
                     source_id: 4,
-                    partition_id: None,
+                    field_id: None,
                     name: "name".to_string(),
                     transform: Transform::Identity,
                 },
@@ -1233,11 +1349,11 @@ mod tests {
             .build()
             .unwrap();
 
-        PartitionSpec::builder(&schema)
+        PartitionSpec::builder(schema)
             .with_spec_id(1)
             .add_unbound_field(UnboundPartitionField {
                 source_id: 1,
-                partition_id: None,
+                field_id: None,
                 name: "id_year".to_string(),
                 transform: Transform::Year,
             })
@@ -1250,7 +1366,7 @@ mod tests {
             .with_spec_id(1)
             .add_partition_fields(vec![UnboundPartitionField {
                 source_id: 1,
-                partition_id: None,
+                field_id: None,
                 name: "id_bucket[16]".to_string(),
                 transform: Transform::Bucket(16),
             }])
@@ -1261,10 +1377,360 @@ mod tests {
             spec_id: Some(1),
             fields: vec![UnboundPartitionField {
                 source_id: 1,
-                partition_id: None,
+                field_id: None,
                 name: "id_bucket[16]".to_string(),
                 transform: Transform::Bucket(16),
             }]
         });
     }
+
+    #[test]
+    fn test_is_compatible_with() {
+        let schema = Schema::builder()
+            .with_fields(vec![
+                NestedField::required(1, "id", Type::Primitive(crate::spec::PrimitiveType::Int))
+                    .into(),
+                NestedField::required(
+                    2,
+                    "name",
+                    Type::Primitive(crate::spec::PrimitiveType::String),
+                )
+                .into(),
+            ])
+            .build()
+            .unwrap();
+
+        let partition_spec_1 = PartitionSpec::builder(schema.clone())
+            .with_spec_id(1)
+            .add_unbound_field(UnboundPartitionField {
+                source_id: 1,
+                field_id: None,
+                name: "id_bucket".to_string(),
+                transform: Transform::Bucket(16),
+            })
+            .unwrap()
+            .build()
+            .unwrap();
+
+        let partition_spec_2 = PartitionSpec::builder(schema)
+            .with_spec_id(1)
+            .add_unbound_field(UnboundPartitionField {
+                source_id: 1,
+                field_id: None,
+                name: "id_bucket".to_string(),
+                transform: Transform::Bucket(16),
+            })
+            .unwrap()
+            .build()
+            .unwrap();
+
+        assert!(partition_spec_1.is_compatible_with(&partition_spec_2));
+    }
+
+    #[test]
+    fn test_not_compatible_with_transform_different() {
+        let schema = Schema::builder()
+            .with_fields(vec![NestedField::required(
+                1,
+                "id",
+                Type::Primitive(crate::spec::PrimitiveType::Int),
+            )
+            .into()])
+            .build()
+            .unwrap();
+
+        let partition_spec_1 = PartitionSpec::builder(schema.clone())
+            .with_spec_id(1)
+            .add_unbound_field(UnboundPartitionField {
+                source_id: 1,
+                field_id: None,
+                name: "id_bucket".to_string(),
+                transform: Transform::Bucket(16),
+            })
+            .unwrap()
+            .build()
+            .unwrap();
+
+        let partition_spec_2 = PartitionSpec::builder(schema)
+            .with_spec_id(1)
+            .add_unbound_field(UnboundPartitionField {
+                source_id: 1,
+                field_id: None,
+                name: "id_bucket".to_string(),
+                transform: Transform::Bucket(32),
+            })
+            .unwrap()
+            .build()
+            .unwrap();
+
+        assert!(!partition_spec_1.is_compatible_with(&partition_spec_2));
+    }
+
+    #[test]
+    fn test_not_compatible_with_source_id_different() {
+        let schema = Schema::builder()
+            .with_fields(vec![
+                NestedField::required(1, "id", Type::Primitive(crate::spec::PrimitiveType::Int))
+                    .into(),
+                NestedField::required(
+                    2,
+                    "name",
+                    Type::Primitive(crate::spec::PrimitiveType::String),
+                )
+                .into(),
+            ])
+            .build()
+            .unwrap();
+
+        let partition_spec_1 = PartitionSpec::builder(schema.clone())
+            .with_spec_id(1)
+            .add_unbound_field(UnboundPartitionField {
+                source_id: 1,
+                field_id: None,
+                name: "id_bucket".to_string(),
+                transform: Transform::Bucket(16),
+            })
+            .unwrap()
+            .build()
+            .unwrap();
+
+        let partition_spec_2 = PartitionSpec::builder(schema)
+            .with_spec_id(1)
+            .add_unbound_field(UnboundPartitionField {
+                source_id: 2,
+                field_id: None,
+                name: "id_bucket".to_string(),
+                transform: Transform::Bucket(16),
+            })
+            .unwrap()
+            .build()
+            .unwrap();
+
+        assert!(!partition_spec_1.is_compatible_with(&partition_spec_2));
+    }
+
+    #[test]
+    fn test_not_compatible_with_order_different() {
+        let schema = Schema::builder()
+            .with_fields(vec![
+                NestedField::required(1, "id", Type::Primitive(crate::spec::PrimitiveType::Int))
+                    .into(),
+                NestedField::required(
+                    2,
+                    "name",
+                    Type::Primitive(crate::spec::PrimitiveType::String),
+                )
+                .into(),
+            ])
+            .build()
+            .unwrap();
+
+        let partition_spec_1 = PartitionSpec::builder(schema.clone())
+            .with_spec_id(1)
+            .add_unbound_field(UnboundPartitionField {
+                source_id: 1,
+                field_id: None,
+                name: "id_bucket".to_string(),
+                transform: Transform::Bucket(16),
+            })
+            .unwrap()
+            .add_unbound_field(UnboundPartitionField {
+                source_id: 2,
+                field_id: None,
+                name: "name".to_string(),
+                transform: Transform::Identity,
+            })
+            .unwrap()
+            .build()
+            .unwrap();
+
+        let partition_spec_2 = PartitionSpec::builder(schema)
+            .with_spec_id(1)
+            .add_unbound_field(UnboundPartitionField {
+                source_id: 2,
+                field_id: None,
+                name: "name".to_string(),
+                transform: Transform::Identity,
+            })
+            .unwrap()
+            .add_unbound_field(UnboundPartitionField {
+                source_id: 1,
+                field_id: None,
+                name: "id_bucket".to_string(),
+                transform: Transform::Bucket(16),
+            })
+            .unwrap()
+            .build()
+            .unwrap();
+
+        assert!(!partition_spec_1.is_compatible_with(&partition_spec_2));
+    }
+
+    #[test]
+    fn test_highest_field_id_unpartitioned() {
+        let spec = PartitionSpec::builder(Schema::builder().with_fields(vec![]).build().unwrap())
+            .with_spec_id(1)
+            .build()
+            .unwrap();
+
+        assert!(spec.highest_field_id().is_none());
+    }
+
+    #[test]
+    fn test_highest_field_id() {
+        let schema = Schema::builder()
+            .with_fields(vec![
+                NestedField::required(1, "id", Type::Primitive(crate::spec::PrimitiveType::Int))
+                    .into(),
+                NestedField::required(
+                    2,
+                    "name",
+                    Type::Primitive(crate::spec::PrimitiveType::String),
+                )
+                .into(),
+            ])
+            .build()
+            .unwrap();
+
+        let spec = PartitionSpec::builder(schema)
+            .with_spec_id(1)
+            .add_unbound_field(UnboundPartitionField {
+                source_id: 1,
+                field_id: Some(1001),
+                name: "id".to_string(),
+                transform: Transform::Identity,
+            })
+            .unwrap()
+            .add_unbound_field(UnboundPartitionField {
+                source_id: 2,
+                field_id: Some(1000),
+                name: "name".to_string(),
+                transform: Transform::Identity,
+            })
+            .unwrap()
+            .build()
+            .unwrap();
+
+        assert_eq!(Some(1001), spec.highest_field_id());
+    }
+
+    #[test]
+    fn test_has_sequential_ids() {
+        let schema = Schema::builder()
+            .with_fields(vec![
+                NestedField::required(1, "id", Type::Primitive(crate::spec::PrimitiveType::Int))
+                    .into(),
+                NestedField::required(
+                    2,
+                    "name",
+                    Type::Primitive(crate::spec::PrimitiveType::String),
+                )
+                .into(),
+            ])
+            .build()
+            .unwrap();
+
+        let spec = PartitionSpec::builder(schema)
+            .with_spec_id(1)
+            .add_unbound_field(UnboundPartitionField {
+                source_id: 1,
+                field_id: Some(1000),
+                name: "id".to_string(),
+                transform: Transform::Identity,
+            })
+            .unwrap()
+            .add_unbound_field(UnboundPartitionField {
+                source_id: 2,
+                field_id: Some(1001),
+                name: "name".to_string(),
+                transform: Transform::Identity,
+            })
+            .unwrap()
+            .build()
+            .unwrap();
+
+        assert_eq!(1000, spec.fields[0].field_id);
+        assert_eq!(1001, spec.fields[1].field_id);
+        assert!(spec.has_sequential_ids());
+    }
+
+    #[test]
+    fn test_sequential_ids_must_start_at_1000() {
+        let schema = Schema::builder()
+            .with_fields(vec![
+                NestedField::required(1, "id", Type::Primitive(crate::spec::PrimitiveType::Int))
+                    .into(),
+                NestedField::required(
+                    2,
+                    "name",
+                    Type::Primitive(crate::spec::PrimitiveType::String),
+                )
+                .into(),
+            ])
+            .build()
+            .unwrap();
+
+        let spec = PartitionSpec::builder(schema)
+            .with_spec_id(1)
+            .add_unbound_field(UnboundPartitionField {
+                source_id: 1,
+                field_id: Some(999),
+                name: "id".to_string(),
+                transform: Transform::Identity,
+            })
+            .unwrap()
+            .add_unbound_field(UnboundPartitionField {
+                source_id: 2,
+                field_id: Some(1000),
+                name: "name".to_string(),
+                transform: Transform::Identity,
+            })
+            .unwrap()
+            .build()
+            .unwrap();
+
+        assert_eq!(999, spec.fields[0].field_id);
+        assert_eq!(1000, spec.fields[1].field_id);
+        assert!(!spec.has_sequential_ids());
+    }
+
+    #[test]
+    fn test_sequential_ids_must_have_no_gaps() {
+        let schema = Schema::builder()
+            .with_fields(vec![
+                NestedField::required(1, "id", Type::Primitive(crate::spec::PrimitiveType::Int))
+                    .into(),
+                NestedField::required(
+                    2,
+                    "name",
+                    Type::Primitive(crate::spec::PrimitiveType::String),
+                )
+                .into(),
+            ])
+            .build()
+            .unwrap();
+
+        let spec = PartitionSpec::builder(schema)
+            .with_spec_id(1)
+            .add_unbound_field(UnboundPartitionField {
+                source_id: 1,
+                field_id: Some(1000),
+                name: "id".to_string(),
+                transform: Transform::Identity,
+            })
+            .unwrap()
+            .add_unbound_field(UnboundPartitionField {
+                source_id: 2,
+                field_id: Some(1002),
+                name: "name".to_string(),
+                transform: Transform::Identity,
+            })
+            .unwrap()
+            .build()
+            .unwrap();
+
+        assert_eq!(1000, spec.fields[0].field_id);
+        assert_eq!(1002, spec.fields[1].field_id);
+        assert!(!spec.has_sequential_ids());
+    }
 }
