@@ -20,7 +20,9 @@ use std::sync::Arc;
 use arrow_arith::arity::binary;
 use arrow_arith::temporal::{date_part, DatePart};
 use arrow_array::types::Date32Type;
-use arrow_array::{Array, ArrayRef, Date32Array, Int32Array, TimestampMicrosecondArray};
+use arrow_array::{
+    Array, ArrayRef, Date32Array, Int32Array, TimestampMicrosecondArray, TimestampNanosecondArray,
+};
 use arrow_schema::{DataType, TimeUnit};
 use chrono::{DateTime, Datelike, Duration};
 
@@ -34,6 +36,8 @@ const HOUR_PER_SECOND: f64 = 1.0_f64 / 3600.0_f64;
 const UNIX_EPOCH_YEAR: i32 = 1970;
 /// One second in micros.
 const MICROS_PER_SECOND: i64 = 1_000_000;
+/// One second in nanos.
+const NANOS_PER_SECOND: i64 = 1_000_000_000;
 
 /// Extract a date or timestamp year, as years from 1970
 #[derive(Debug)]
@@ -41,7 +45,7 @@ pub struct Year;
 
 impl Year {
     #[inline]
-    fn timestamp_to_year(timestamp: i64) -> Result<i32> {
+    fn timestamp_to_year_micros(timestamp: i64) -> Result<i32> {
         Ok(DateTime::from_timestamp_micros(timestamp)
             .ok_or_else(|| {
                 Error::new(
@@ -52,6 +56,11 @@ impl Year {
             .year()
             - UNIX_EPOCH_YEAR)
     }
+
+    #[inline]
+    fn timestamp_to_year_nanos(timestamp: i64) -> Result<i32> {
+        Ok(DateTime::from_timestamp_nanos(timestamp).year() - UNIX_EPOCH_YEAR)
+    }
 }
 
 impl TransformFunction for Year {
@@ -72,8 +81,18 @@ impl TransformFunction for Year {
             (PrimitiveType::Date, PrimitiveLiteral::Int(v)) => {
                 Date32Type::to_naive_date(*v).year() - UNIX_EPOCH_YEAR
             }
-            (PrimitiveType::Timestamp, PrimitiveLiteral::Long(v)) => Self::timestamp_to_year(*v)?,
-            (PrimitiveType::Timestamptz, PrimitiveLiteral::Long(v)) => Self::timestamp_to_year(*v)?,
+            (PrimitiveType::Timestamp, PrimitiveLiteral::Long(v)) => {
+                Self::timestamp_to_year_micros(*v)?
+            }
+            (PrimitiveType::Timestamptz, PrimitiveLiteral::Long(v)) => {
+                Self::timestamp_to_year_micros(*v)?
+            }
+            (PrimitiveType::TimestampNs, PrimitiveLiteral::Long(v)) => {
+                Self::timestamp_to_year_nanos(*v)?
+            }
+            (PrimitiveType::TimestamptzNs, PrimitiveLiteral::Long(v)) => {
+                Self::timestamp_to_year_nanos(*v)?
+            }
             _ => {
                 return Err(crate::Error::new(
                     crate::ErrorKind::FeatureUnsupported,
@@ -94,7 +113,7 @@ pub struct Month;
 
 impl Month {
     #[inline]
-    fn timestamp_to_month(timestamp: i64) -> Result<i32> {
+    fn timestamp_to_month_micros(timestamp: i64) -> Result<i32> {
         // date: aaaa-aa-aa
         // unix epoch date: 1970-01-01
         // if date > unix epoch date, delta month = (aa - 1) + 12 * (aaaa-1970)
@@ -114,6 +133,22 @@ impl Month {
             Ok(-delta)
         }
     }
+
+    #[inline]
+    fn timestamp_to_month_nanos(timestamp: i64) -> Result<i32> {
+        // date: aaaa-aa-aa
+        // unix epoch date: 1970-01-01
+        // if date > unix epoch date, delta month = (aa - 1) + 12 * (aaaa-1970)
+        // if date < unix epoch date, delta month = (12 - (aa - 1)) + 12 * (1970-aaaa-1)
+        let date = DateTime::from_timestamp_nanos(timestamp);
+        let unix_epoch_date = DateTime::from_timestamp_nanos(0);
+        if date > unix_epoch_date {
+            Ok((date.month0() as i32) + 12 * (date.year() - UNIX_EPOCH_YEAR))
+        } else {
+            let delta = (12 - date.month0() as i32) + 12 * (UNIX_EPOCH_YEAR - date.year() - 1);
+            Ok(-delta)
+        }
+    }
 }
 
 impl TransformFunction for Month {
@@ -144,9 +179,17 @@ impl TransformFunction for Month {
                 (Date32Type::to_naive_date(*v).year() - UNIX_EPOCH_YEAR) * 12
                     + Date32Type::to_naive_date(*v).month0() as i32
             }
-            (PrimitiveType::Timestamp, PrimitiveLiteral::Long(v)) => Self::timestamp_to_month(*v)?,
+            (PrimitiveType::Timestamp, PrimitiveLiteral::Long(v)) => {
+                Self::timestamp_to_month_micros(*v)?
+            }
             (PrimitiveType::Timestamptz, PrimitiveLiteral::Long(v)) => {
-                Self::timestamp_to_month(*v)?
+                Self::timestamp_to_month_micros(*v)?
+            }
+            (PrimitiveType::TimestampNs, PrimitiveLiteral::Long(v)) => {
+                Self::timestamp_to_month_nanos(*v)?
+            }
+            (PrimitiveType::TimestamptzNs, PrimitiveLiteral::Long(v)) => {
+                Self::timestamp_to_month_nanos(*v)?
             }
             _ => {
                 return Err(crate::Error::new(
@@ -196,16 +239,50 @@ impl Day {
 
         Ok(days)
     }
+
+    fn day_timestamp_nano(v: i64) -> Result<i32> {
+        let secs = v / NANOS_PER_SECOND;
+
+        let (nanos, offset) = if v >= 0 {
+            let nanos = (v.rem_euclid(NANOS_PER_SECOND)) as u32;
+            let offset = 0i64;
+            (nanos, offset)
+        } else {
+            let v = v + 1;
+            let nanos = (v.rem_euclid(NANOS_PER_SECOND)) as u32;
+            let offset = 1i64;
+            (nanos, offset)
+        };
+
+        let delta = Duration::new(secs, nanos).ok_or_else(|| {
+            Error::new(
+                ErrorKind::DataInvalid,
+                format!(
+                    "Failed to create 'TimeDelta' from seconds {} and nanos {}",
+                    secs, nanos
+                ),
+            )
+        })?;
+
+        let days = (delta.num_days() - offset) as i32;
+
+        Ok(days)
+    }
 }
 
 impl TransformFunction for Day {
     fn transform(&self, input: ArrayRef) -> Result<ArrayRef> {
-        let res: Int32Array = match input.data_type() {
+        let res: Date32Array = match input.data_type() {
             DataType::Timestamp(TimeUnit::Microsecond, _) => input
                 .as_any()
                 .downcast_ref::<TimestampMicrosecondArray>()
                 .unwrap()
                 .try_unary(|v| -> Result<i32> { Self::day_timestamp_micro(v) })?,
+            DataType::Timestamp(TimeUnit::Nanosecond, _) => input
+                .as_any()
+                .downcast_ref::<TimestampNanosecondArray>()
+                .unwrap()
+                .try_unary(|v| -> Result<i32> { Self::day_timestamp_nano(v) })?,
             DataType::Date32 => input
                 .as_any()
                 .downcast_ref::<Date32Array>()
@@ -231,6 +308,12 @@ impl TransformFunction for Day {
             (PrimitiveType::Timestamptz, PrimitiveLiteral::Long(v)) => {
                 Self::day_timestamp_micro(*v)?
             }
+            (PrimitiveType::TimestampNs, PrimitiveLiteral::Long(v)) => {
+                Self::day_timestamp_nano(*v)?
+            }
+            (PrimitiveType::TimestamptzNs, PrimitiveLiteral::Long(v)) => {
+                Self::day_timestamp_nano(*v)?
+            }
             _ => {
                 return Err(crate::Error::new(
                     crate::ErrorKind::FeatureUnsupported,
@@ -241,7 +324,7 @@ impl TransformFunction for Day {
                 ))
             }
         };
-        Ok(Some(Datum::int(val)))
+        Ok(Some(Datum::date(val)))
     }
 }
 
@@ -254,6 +337,11 @@ impl Hour {
     fn hour_timestamp_micro(v: i64) -> i32 {
         (v as f64 / 1000.0 / 1000.0 * HOUR_PER_SECOND) as i32
     }
+
+    #[inline]
+    fn hour_timestamp_nano(v: i64) -> i32 {
+        (v as f64 / 1_000_000.0 / 1000.0 * HOUR_PER_SECOND) as i32
+    }
 }
 
 impl TransformFunction for Hour {
@@ -283,6 +371,12 @@ impl TransformFunction for Hour {
             (PrimitiveType::Timestamptz, PrimitiveLiteral::Long(v)) => {
                 Self::hour_timestamp_micro(*v)
             }
+            (PrimitiveType::TimestampNs, PrimitiveLiteral::Long(v)) => {
+                Self::hour_timestamp_nano(*v)
+            }
+            (PrimitiveType::TimestamptzNs, PrimitiveLiteral::Long(v)) => {
+                Self::hour_timestamp_nano(*v)
+            }
             _ => {
                 return Err(crate::Error::new(
                     crate::ErrorKind::FeatureUnsupported,
@@ -307,7 +401,7 @@ mod test {
     use crate::expr::PredicateOperator;
     use crate::spec::PrimitiveType::{
         Binary, Date, Decimal, Fixed, Int, Long, String as StringType, Time, Timestamp,
-        Timestamptz, Uuid,
+        TimestampNs, Timestamptz, TimestamptzNs, Uuid,
     };
     use crate::spec::Type::{Primitive, Struct};
     use crate::spec::{Datum, NestedField, PrimitiveType, StructType, Transform, Type};
@@ -334,7 +428,7 @@ mod test {
             ],
             trans_types: vec![
                 (Primitive(Binary), None),
-                (Primitive(Date), Some(Primitive(Date))),
+                (Primitive(Date), Some(Primitive(Int))),
                 (
                     Primitive(Decimal {
                         precision: 8,
@@ -348,8 +442,10 @@ mod test {
                 (Primitive(StringType), None),
                 (Primitive(Uuid), None),
                 (Primitive(Time), None),
-                (Primitive(Timestamp), Some(Primitive(Date))),
-                (Primitive(Timestamptz), Some(Primitive(Date))),
+                (Primitive(Timestamp), Some(Primitive(Int))),
+                (Primitive(Timestamptz), Some(Primitive(Int))),
+                (Primitive(TimestampNs), Some(Primitive(Int))),
+                (Primitive(TimestamptzNs), Some(Primitive(Int))),
                 (
                     Struct(StructType::new(vec![NestedField::optional(
                         1,
@@ -384,7 +480,7 @@ mod test {
             ],
             trans_types: vec![
                 (Primitive(Binary), None),
-                (Primitive(Date), Some(Primitive(Date))),
+                (Primitive(Date), Some(Primitive(Int))),
                 (
                     Primitive(Decimal {
                         precision: 8,
@@ -398,8 +494,10 @@ mod test {
                 (Primitive(StringType), None),
                 (Primitive(Uuid), None),
                 (Primitive(Time), None),
-                (Primitive(Timestamp), Some(Primitive(Date))),
-                (Primitive(Timestamptz), Some(Primitive(Date))),
+                (Primitive(Timestamp), Some(Primitive(Int))),
+                (Primitive(Timestamptz), Some(Primitive(Int))),
+                (Primitive(TimestampNs), Some(Primitive(Int))),
+                (Primitive(TimestamptzNs), Some(Primitive(Int))),
                 (
                     Struct(StructType::new(vec![NestedField::optional(
                         1,
@@ -450,6 +548,8 @@ mod test {
                 (Primitive(Time), None),
                 (Primitive(Timestamp), Some(Primitive(Date))),
                 (Primitive(Timestamptz), Some(Primitive(Date))),
+                (Primitive(TimestampNs), Some(Primitive(Date))),
+                (Primitive(TimestamptzNs), Some(Primitive(Date))),
                 (
                     Struct(StructType::new(vec![NestedField::optional(
                         1,
@@ -500,6 +600,8 @@ mod test {
                 (Primitive(Time), None),
                 (Primitive(Timestamp), Some(Primitive(Int))),
                 (Primitive(Timestamptz), Some(Primitive(Int))),
+                (Primitive(TimestampNs), Some(Primitive(Int))),
+                (Primitive(TimestamptzNs), Some(Primitive(Int))),
                 (
                     Struct(StructType::new(vec![NestedField::optional(
                         1,
@@ -1112,7 +1214,7 @@ mod test {
                 PredicateOperator::LessThan,
                 Datum::timestamp_from_str(value)?,
             ),
-            Some("name <= 0"),
+            Some("name <= 1970-01-01"),
         )?;
 
         fixture.assert_projection(
@@ -1120,7 +1222,7 @@ mod test {
                 PredicateOperator::LessThanOrEq,
                 Datum::timestamp_from_str(value)?,
             ),
-            Some("name <= 0"),
+            Some("name <= 1970-01-01"),
         )?;
 
         fixture.assert_projection(
@@ -1128,7 +1230,7 @@ mod test {
                 PredicateOperator::GreaterThan,
                 Datum::timestamp_from_str(value)?,
             ),
-            Some("name >= 0"),
+            Some("name >= 1970-01-01"),
         )?;
 
         fixture.assert_projection(
@@ -1136,12 +1238,12 @@ mod test {
                 PredicateOperator::GreaterThanOrEq,
                 Datum::timestamp_from_str(value)?,
             ),
-            Some("name >= -1"),
+            Some("name >= 1969-12-31"),
         )?;
 
         fixture.assert_projection(
             &fixture.binary_predicate(PredicateOperator::Eq, Datum::timestamp_from_str(value)?),
-            Some("name IN (-1, 0)"),
+            Some("name IN (1969-12-31, 1970-01-01)"),
         )?;
 
         fixture.assert_projection(
@@ -1154,7 +1256,7 @@ mod test {
                 Datum::timestamp_from_str(value)?,
                 Datum::timestamp_from_str(another)?,
             ]),
-            Some("name IN (0, -1)"),
+            Some("name IN (1970-01-01, 1969-12-31)"),
         )?;
 
         fixture.assert_projection(
@@ -1186,7 +1288,7 @@ mod test {
                 PredicateOperator::LessThan,
                 Datum::timestamp_from_str(value)?,
             ),
-            Some("name <= 17501"),
+            Some("name <= 2017-12-01"),
         )?;
 
         fixture.assert_projection(
@@ -1194,7 +1296,7 @@ mod test {
                 PredicateOperator::LessThanOrEq,
                 Datum::timestamp_from_str(value)?,
             ),
-            Some("name <= 17501"),
+            Some("name <= 2017-12-01"),
         )?;
 
         fixture.assert_projection(
@@ -1202,7 +1304,7 @@ mod test {
                 PredicateOperator::GreaterThan,
                 Datum::timestamp_from_str(value)?,
             ),
-            Some("name >= 17502"),
+            Some("name >= 2017-12-02"),
         )?;
 
         fixture.assert_projection(
@@ -1210,12 +1312,12 @@ mod test {
                 PredicateOperator::GreaterThanOrEq,
                 Datum::timestamp_from_str(value)?,
             ),
-            Some("name >= 17501"),
+            Some("name >= 2017-12-01"),
         )?;
 
         fixture.assert_projection(
             &fixture.binary_predicate(PredicateOperator::Eq, Datum::timestamp_from_str(value)?),
-            Some("name = 17501"),
+            Some("name = 2017-12-01"),
         )?;
 
         fixture.assert_projection(
@@ -1228,7 +1330,7 @@ mod test {
                 Datum::timestamp_from_str(value)?,
                 Datum::timestamp_from_str(another)?,
             ]),
-            Some("name IN (17501, 17502)"),
+            Some("name IN (2017-12-02, 2017-12-01)"),
         )?;
 
         fixture.assert_projection(
@@ -1260,7 +1362,7 @@ mod test {
                 PredicateOperator::LessThan,
                 Datum::timestamp_from_str(value)?,
             ),
-            Some("name <= -365"),
+            Some("name <= 1969-01-01"),
         )?;
 
         fixture.assert_projection(
@@ -1268,7 +1370,7 @@ mod test {
                 PredicateOperator::LessThanOrEq,
                 Datum::timestamp_from_str(value)?,
             ),
-            Some("name <= -364"),
+            Some("name <= 1969-01-02"),
         )?;
 
         fixture.assert_projection(
@@ -1276,7 +1378,7 @@ mod test {
                 PredicateOperator::GreaterThan,
                 Datum::timestamp_from_str(value)?,
             ),
-            Some("name >= -365"),
+            Some("name >= 1969-01-01"),
         )?;
 
         fixture.assert_projection(
@@ -1284,12 +1386,12 @@ mod test {
                 PredicateOperator::GreaterThanOrEq,
                 Datum::timestamp_from_str(value)?,
             ),
-            Some("name >= -365"),
+            Some("name >= 1969-01-01"),
         )?;
 
         fixture.assert_projection(
             &fixture.binary_predicate(PredicateOperator::Eq, Datum::timestamp_from_str(value)?),
-            Some("name IN (-364, -365)"),
+            Some("name IN (1969-01-01, 1969-01-02)"),
         )?;
 
         fixture.assert_projection(
@@ -1302,7 +1404,7 @@ mod test {
                 Datum::timestamp_from_str(value)?,
                 Datum::timestamp_from_str(another)?,
             ]),
-            Some("name IN (-363, -365, -364)"),
+            Some("name IN (1969-01-02, 1969-01-01, 1969-01-03)"),
         )?;
 
         fixture.assert_projection(
@@ -1334,7 +1436,7 @@ mod test {
                 PredicateOperator::LessThan,
                 Datum::timestamp_from_str(value)?,
             ),
-            Some("name <= 17500"),
+            Some("name <= 2017-11-30"),
         )?;
 
         fixture.assert_projection(
@@ -1342,7 +1444,7 @@ mod test {
                 PredicateOperator::LessThanOrEq,
                 Datum::timestamp_from_str(value)?,
             ),
-            Some("name <= 17501"),
+            Some("name <= 2017-12-01"),
         )?;
 
         fixture.assert_projection(
@@ -1350,7 +1452,7 @@ mod test {
                 PredicateOperator::GreaterThan,
                 Datum::timestamp_from_str(value)?,
             ),
-            Some("name >= 17501"),
+            Some("name >= 2017-12-01"),
         )?;
 
         fixture.assert_projection(
@@ -1358,12 +1460,12 @@ mod test {
                 PredicateOperator::GreaterThanOrEq,
                 Datum::timestamp_from_str(value)?,
             ),
-            Some("name >= 17501"),
+            Some("name >= 2017-12-01"),
         )?;
 
         fixture.assert_projection(
             &fixture.binary_predicate(PredicateOperator::Eq, Datum::timestamp_from_str(value)?),
-            Some("name = 17501"),
+            Some("name = 2017-12-01"),
         )?;
 
         fixture.assert_projection(
@@ -1376,7 +1478,7 @@ mod test {
                 Datum::timestamp_from_str(value)?,
                 Datum::timestamp_from_str(another)?,
             ]),
-            Some("name IN (17501, 17502)"),
+            Some("name IN (2017-12-02, 2017-12-01)"),
         )?;
 
         fixture.assert_projection(
@@ -1408,7 +1510,7 @@ mod test {
                 PredicateOperator::LessThan,
                 Datum::timestamp_from_str(value)?,
             ),
-            Some("name <= 0"),
+            Some("name <= 1970-01-01"),
         )?;
 
         fixture.assert_projection(
@@ -1416,7 +1518,7 @@ mod test {
                 PredicateOperator::LessThanOrEq,
                 Datum::timestamp_from_str(value)?,
             ),
-            Some("name <= 0"),
+            Some("name <= 1970-01-01"),
         )?;
 
         fixture.assert_projection(
@@ -1424,7 +1526,7 @@ mod test {
                 PredicateOperator::GreaterThan,
                 Datum::timestamp_from_str(value)?,
             ),
-            Some("name >= 0"),
+            Some("name >= 1970-01-01"),
         )?;
 
         fixture.assert_projection(
@@ -1432,12 +1534,12 @@ mod test {
                 PredicateOperator::GreaterThanOrEq,
                 Datum::timestamp_from_str(value)?,
             ),
-            Some("name >= 0"),
+            Some("name >= 1970-01-01"),
         )?;
 
         fixture.assert_projection(
             &fixture.binary_predicate(PredicateOperator::Eq, Datum::timestamp_from_str(value)?),
-            Some("name = 0"),
+            Some("name = 1970-01-01"),
         )?;
 
         fixture.assert_projection(
@@ -1450,7 +1552,7 @@ mod test {
                 Datum::timestamp_from_str(value)?,
                 Datum::timestamp_from_str(another)?,
             ]),
-            Some("name IN (1, 0)"),
+            Some("name IN (1970-01-01, 1970-01-02)"),
         )?;
 
         fixture.assert_projection(
@@ -1479,7 +1581,7 @@ mod test {
 
         fixture.assert_projection(
             &fixture.binary_predicate(PredicateOperator::LessThan, Datum::date_from_str(value)?),
-            Some("name <= -3"),
+            Some("name <= 1969-12-29"),
         )?;
 
         fixture.assert_projection(
@@ -1487,12 +1589,12 @@ mod test {
                 PredicateOperator::LessThanOrEq,
                 Datum::date_from_str(value)?,
             ),
-            Some("name <= -2"),
+            Some("name <= 1969-12-30"),
         )?;
 
         fixture.assert_projection(
             &fixture.binary_predicate(PredicateOperator::GreaterThan, Datum::date_from_str(value)?),
-            Some("name >= -1"),
+            Some("name >= 1969-12-31"),
         )?;
 
         fixture.assert_projection(
@@ -1500,12 +1602,12 @@ mod test {
                 PredicateOperator::GreaterThanOrEq,
                 Datum::date_from_str(value)?,
             ),
-            Some("name >= -2"),
+            Some("name >= 1969-12-30"),
         )?;
 
         fixture.assert_projection(
             &fixture.binary_predicate(PredicateOperator::Eq, Datum::date_from_str(value)?),
-            Some("name = -2"),
+            Some("name = 1969-12-30"),
         )?;
 
         fixture.assert_projection(
@@ -1518,7 +1620,7 @@ mod test {
                 Datum::date_from_str(value)?,
                 Datum::date_from_str(another)?,
             ]),
-            Some("name IN (-2, -4)"),
+            Some("name IN (1969-12-28, 1969-12-30)"),
         )?;
 
         fixture.assert_projection(
@@ -1547,7 +1649,7 @@ mod test {
 
         fixture.assert_projection(
             &fixture.binary_predicate(PredicateOperator::LessThan, Datum::date_from_str(value)?),
-            Some("name <= 17166"),
+            Some("name <= 2016-12-31"),
         )?;
 
         fixture.assert_projection(
@@ -1555,12 +1657,12 @@ mod test {
                 PredicateOperator::LessThanOrEq,
                 Datum::date_from_str(value)?,
             ),
-            Some("name <= 17167"),
+            Some("name <= 2017-01-01"),
         )?;
 
         fixture.assert_projection(
             &fixture.binary_predicate(PredicateOperator::GreaterThan, Datum::date_from_str(value)?),
-            Some("name >= 17168"),
+            Some("name >= 2017-01-02"),
         )?;
 
         fixture.assert_projection(
@@ -1568,12 +1670,12 @@ mod test {
                 PredicateOperator::GreaterThanOrEq,
                 Datum::date_from_str(value)?,
             ),
-            Some("name >= 17167"),
+            Some("name >= 2017-01-01"),
         )?;
 
         fixture.assert_projection(
             &fixture.binary_predicate(PredicateOperator::Eq, Datum::date_from_str(value)?),
-            Some("name = 17167"),
+            Some("name = 2017-01-01"),
         )?;
 
         fixture.assert_projection(
@@ -1586,7 +1688,7 @@ mod test {
                 Datum::date_from_str(value)?,
                 Datum::date_from_str(another)?,
             ]),
-            Some("name IN (17531, 17167)"),
+            Some("name IN (2017-01-01, 2017-12-31)"),
         )?;
 
         fixture.assert_projection(
@@ -2323,6 +2425,50 @@ mod test {
         assert_eq!(res, expect);
     }
 
+    fn test_timestamp_ns_and_tz_transform(
+        time: &str,
+        transform: &BoxedTransformFunction,
+        expect: Datum,
+    ) {
+        let timestamp_ns = Datum::timestamp_nanos(
+            NaiveDateTime::parse_from_str(time, "%Y-%m-%d %H:%M:%S.%f")
+                .unwrap()
+                .and_utc()
+                .timestamp_nanos_opt()
+                .unwrap(),
+        );
+        let timestamptz_ns = Datum::timestamptz_nanos(
+            NaiveDateTime::parse_from_str(time, "%Y-%m-%d %H:%M:%S.%f")
+                .unwrap()
+                .and_utc()
+                .timestamp_nanos_opt()
+                .unwrap(),
+        );
+        let res = transform.transform_literal(&timestamp_ns).unwrap().unwrap();
+        assert_eq!(res, expect);
+        let res = transform
+            .transform_literal(&timestamptz_ns)
+            .unwrap()
+            .unwrap();
+        assert_eq!(res, expect);
+    }
+
+    fn test_timestamp_ns_and_tz_transform_using_i64(
+        time: i64,
+        transform: &BoxedTransformFunction,
+        expect: Datum,
+    ) {
+        let timestamp_ns = Datum::timestamp_nanos(time);
+        let timestamptz_ns = Datum::timestamptz_nanos(time);
+        let res = transform.transform_literal(&timestamp_ns).unwrap().unwrap();
+        assert_eq!(res, expect);
+        let res = transform
+            .transform_literal(&timestamptz_ns)
+            .unwrap()
+            .unwrap();
+        assert_eq!(res, expect);
+    }
+
     #[test]
     fn test_transform_year_literal() {
         let year = Box::new(super::Year) as BoxedTransformFunction;
@@ -2338,6 +2484,14 @@ mod test {
             Datum::int(1970 - super::UNIX_EPOCH_YEAR),
         );
         test_timestamp_and_tz_transform("1969-01-01 00:00:00.00", &year, Datum::int(-1));
+
+        // Test TimestampNanosecond
+        test_timestamp_ns_and_tz_transform_using_i64(
+            186280000000,
+            &year,
+            Datum::int(1970 - super::UNIX_EPOCH_YEAR),
+        );
+        test_timestamp_ns_and_tz_transform("1969-01-01 00:00:00.00", &year, Datum::int(-1));
     }
 
     #[test]
@@ -2432,6 +2586,17 @@ mod test {
         test_timestamp_and_tz_transform("2017-12-01 00:00:00.00", &month, Datum::int(575));
         test_timestamp_and_tz_transform("1970-01-01 00:00:00.00", &month, Datum::int(0));
         test_timestamp_and_tz_transform("1969-12-31 00:00:00.00", &month, Datum::int(-1));
+
+        // Test TimestampNanosecond
+        test_timestamp_ns_and_tz_transform_using_i64(
+            186280000000,
+            &month,
+            Datum::int((1970 - super::UNIX_EPOCH_YEAR) * 12),
+        );
+        test_timestamp_ns_and_tz_transform("1969-12-01 23:00:00.00", &month, Datum::int(-1));
+        test_timestamp_ns_and_tz_transform("2017-12-01 00:00:00.00", &month, Datum::int(575));
+        test_timestamp_ns_and_tz_transform("1970-01-01 00:00:00.00", &month, Datum::int(0));
+        test_timestamp_ns_and_tz_transform("1969-12-31 00:00:00.00", &month, Datum::int(-1));
     }
 
     #[test]
@@ -2464,7 +2629,7 @@ mod test {
                 .collect::<Vec<i32>>(),
         ));
         let res = day.transform(date_array).unwrap();
-        let res = res.as_any().downcast_ref::<Int32Array>().unwrap();
+        let res = res.as_any().downcast_ref::<Date32Array>().unwrap();
         assert_eq!(res.len(), 5);
         assert_eq!(res.value(0), expect_day[0]);
         assert_eq!(res.value(1), expect_day[1]);
@@ -2503,7 +2668,7 @@ mod test {
                 .collect::<Vec<i64>>(),
         ));
         let res = day.transform(date_array).unwrap();
-        let res = res.as_any().downcast_ref::<Int32Array>().unwrap();
+        let res = res.as_any().downcast_ref::<Date32Array>().unwrap();
         assert_eq!(res.len(), 5);
         assert_eq!(res.value(0), expect_day[0]);
         assert_eq!(res.value(1), expect_day[1]);
@@ -2516,13 +2681,18 @@ mod test {
     fn test_transform_days_literal() {
         let day = Box::new(super::Day) as BoxedTransformFunction;
         // Test Date32
-        test_date(18628, &day, Datum::int(18628));
-        test_date(-31, &day, Datum::int(-31));
+        test_date(18628, &day, Datum::date(18628));
+        test_date(-31, &day, Datum::date(-31));
 
         // Test TimestampMicrosecond
-        test_timestamp_and_tz_transform_using_i64(1512151975038194, &day, Datum::int(17501));
-        test_timestamp_and_tz_transform_using_i64(-115200000000, &day, Datum::int(-2));
-        test_timestamp_and_tz_transform("2017-12-01 10:30:42.123", &day, Datum::int(17501));
+        test_timestamp_and_tz_transform_using_i64(1512151975038194, &day, Datum::date(17501));
+        test_timestamp_and_tz_transform_using_i64(-115200000000, &day, Datum::date(-2));
+        test_timestamp_and_tz_transform("2017-12-01 10:30:42.123", &day, Datum::date(17501));
+
+        // Test TimestampNanosecond
+        test_timestamp_ns_and_tz_transform_using_i64(1512151975038194, &day, Datum::date(17));
+        test_timestamp_ns_and_tz_transform_using_i64(-115200000000, &day, Datum::date(-1));
+        test_timestamp_ns_and_tz_transform("2017-12-01 10:30:42.123", &day, Datum::date(17501));
     }
 
     #[test]
@@ -2591,5 +2761,9 @@ mod test {
         // Test TimestampMicrosecond
         test_timestamp_and_tz_transform("2017-12-01 18:00:00.00", &hour, Datum::int(420042));
         test_timestamp_and_tz_transform("1969-12-31 23:00:00.00", &hour, Datum::int(-1));
+
+        // Test TimestampNanosecond
+        test_timestamp_ns_and_tz_transform("2017-12-01 18:00:00.00", &hour, Datum::int(420042));
+        test_timestamp_ns_and_tz_transform("1969-12-31 23:00:00.00", &hour, Datum::int(-1));
     }
 }
