@@ -174,7 +174,7 @@ mod test {
     use crate::expr::PredicateOperator;
     use crate::spec::PrimitiveType::{
         Binary, Date, Decimal, Fixed, Int, Long, String as StringType, Time, Timestamp,
-        Timestamptz, Uuid,
+        TimestampNs, Timestamptz, TimestamptzNs, Uuid,
     };
     use crate::spec::Type::{Primitive, Struct};
     use crate::spec::{Datum, NestedField, PrimitiveType, StructType, Transform, Type};
@@ -219,6 +219,8 @@ mod test {
                 (Primitive(Time), None),
                 (Primitive(Timestamp), None),
                 (Primitive(Timestamptz), None),
+                (Primitive(TimestampNs), None),
+                (Primitive(TimestamptzNs), None),
                 (
                     Struct(StructType::new(vec![NestedField::optional(
                         1,
@@ -710,11 +712,11 @@ mod test {
         );
 
         // test decimal
-        let mut buidler = PrimitiveBuilder::<Decimal128Type>::new()
+        let mut builder = PrimitiveBuilder::<Decimal128Type>::new()
             .with_precision_and_scale(20, 2)
             .unwrap();
-        buidler.append_value(1065);
-        let input = Arc::new(buidler.finish());
+        builder.append_value(1065);
+        let input = Arc::new(builder.finish());
         let res = super::Truncate::new(50).transform(input).unwrap();
         assert_eq!(
             res.as_any()
