@@ -27,29 +27,26 @@ use futures::future::BoxFuture;
 use itertools::Itertools;
 use parquet::arrow::async_writer::AsyncFileWriter as ArrowAsyncFileWriter;
 use parquet::arrow::AsyncArrowWriter;
-use parquet::data_type::{
-    BoolType, ByteArray, ByteArrayType, DataType as ParquetDataType, DoubleType, FixedLenByteArray,
-    FixedLenByteArrayType, FloatType, Int32Type, Int64Type,
-};
 use parquet::file::properties::WriterProperties;
-use parquet::file::statistics::{from_thrift, Statistics, TypedStatistics};
+use parquet::file::statistics::{from_thrift, Statistics};
 use parquet::format::FileMetaData;
-use uuid::Uuid;
 
 use super::location_generator::{FileNameGenerator, LocationGenerator};
 use super::track_writer::TrackWriter;
 use super::{FileWriter, FileWriterBuilder};
-use crate::arrow::DEFAULT_MAP_FIELD_NAME;
+use crate::arrow::{
+    get_parquet_stat_max_as_datum, get_parquet_stat_min_as_datum, DEFAULT_MAP_FIELD_NAME,
+};
 use crate::io::{FileIO, FileWrite, OutputFile};
 use crate::spec::{
     visit_schema, DataFileBuilder, DataFileFormat, Datum, ListType, MapType, NestedFieldRef,
-    PrimitiveLiteral, PrimitiveType, Schema, SchemaRef, SchemaVisitor, StructType, Type,
+    PrimitiveType, Schema, SchemaRef, SchemaVisitor, StructType, Type,
 };
 use crate::writer::CurrentFileStatus;
 use crate::{Error, ErrorKind, Result};
 
 /// ParquetWriterBuilder is used to builder a [`ParquetWriter`]
-#[derive(Clone)]
+#[derive(Clone, Debug)]
 pub struct ParquetWriterBuilder<T: LocationGenerator, F: FileNameGenerator> {
     props: WriterProperties,
     schema: SchemaRef,
@@ -237,34 +234,26 @@ impl MinMaxColAggregator {
         }
     }
 
-    fn update_state<T: ParquetDataType>(
-        &mut self,
-        field_id: i32,
-        state: &TypedStatistics<T>,
-        convert_func: impl Fn(<T as ParquetDataType>::T) -> Result<Datum>,
-    ) {
-        if state.min_is_exact() {
-            let val = convert_func(state.min().clone()).unwrap();
-            self.lower_bounds
-                .entry(field_id)
-                .and_modify(|e| {
-                    if *e > val {
-                        *e = val.clone()
-                    }
-                })
-                .or_insert(val);
-        }
-        if state.max_is_exact() {
-            let val = convert_func(state.max().clone()).unwrap();
-            self.upper_bounds
-                .entry(field_id)
-                .and_modify(|e| {
-                    if *e < val {
-                        *e = val.clone()
-                    }
-                })
-                .or_insert(val);
-        }
+    fn update_state_min(&mut self, field_id: i32, datum: Datum) {
+        self.lower_bounds
+            .entry(field_id)
+            .and_modify(|e| {
+                if *e > datum {
+                    *e = datum.clone()
+                }
+            })
+            .or_insert(datum);
+    }
+
+    fn update_state_max(&mut self, field_id: i32, datum: Datum) {
+        self.upper_bounds
+            .entry(field_id)
+            .and_modify(|e| {
+                if *e > datum {
+                    *e = datum.clone()
+                }
+            })
+            .or_insert(datum);
     }
 
     fn update(&mut self, field_id: i32, value: Statistics) -> Result<()> {
@@ -287,134 +276,28 @@ impl MinMaxColAggregator {
             ));
         };
 
-        match (&ty, value) {
-            (PrimitiveType::Boolean, Statistics::Boolean(stat)) => {
-                let convert_func = |v: bool| Result::<Datum>::Ok(Datum::bool(v));
-                self.update_state::<BoolType>(field_id, &stat, convert_func)
-            }
-            (PrimitiveType::Int, Statistics::Int32(stat)) => {
-                let convert_func = |v: i32| Result::<Datum>::Ok(Datum::int(v));
-                self.update_state::<Int32Type>(field_id, &stat, convert_func)
-            }
-            (PrimitiveType::Long, Statistics::Int64(stat)) => {
-                let convert_func = |v: i64| Result::<Datum>::Ok(Datum::long(v));
-                self.update_state::<Int64Type>(field_id, &stat, convert_func)
-            }
-            (PrimitiveType::Float, Statistics::Float(stat)) => {
-                let convert_func = |v: f32| Result::<Datum>::Ok(Datum::float(v));
-                self.update_state::<FloatType>(field_id, &stat, convert_func)
-            }
-            (PrimitiveType::Double, Statistics::Double(stat)) => {
-                let convert_func = |v: f64| Result::<Datum>::Ok(Datum::double(v));
-                self.update_state::<DoubleType>(field_id, &stat, convert_func)
-            }
-            (PrimitiveType::String, Statistics::ByteArray(stat)) => {
-                let convert_func = |v: ByteArray| {
-                    Result::<Datum>::Ok(Datum::string(
-                        String::from_utf8(v.data().to_vec()).unwrap(),
-                    ))
-                };
-                self.update_state::<ByteArrayType>(field_id, &stat, convert_func)
-            }
-            (PrimitiveType::Binary, Statistics::ByteArray(stat)) => {
-                let convert_func =
-                    |v: ByteArray| Result::<Datum>::Ok(Datum::binary(v.data().to_vec()));
-                self.update_state::<ByteArrayType>(field_id, &stat, convert_func)
-            }
-            (PrimitiveType::Date, Statistics::Int32(stat)) => {
-                let convert_func = |v: i32| Result::<Datum>::Ok(Datum::date(v));
-                self.update_state::<Int32Type>(field_id, &stat, convert_func)
-            }
-            (PrimitiveType::Time, Statistics::Int64(stat)) => {
-                let convert_func = |v: i64| Datum::time_micros(v);
-                self.update_state::<Int64Type>(field_id, &stat, convert_func)
-            }
-            (PrimitiveType::Timestamp, Statistics::Int64(stat)) => {
-                let convert_func = |v: i64| Result::<Datum>::Ok(Datum::timestamp_micros(v));
-                self.update_state::<Int64Type>(field_id, &stat, convert_func)
-            }
-            (PrimitiveType::Timestamptz, Statistics::Int64(stat)) => {
-                let convert_func = |v: i64| Result::<Datum>::Ok(Datum::timestamptz_micros(v));
-                self.update_state::<Int64Type>(field_id, &stat, convert_func)
-            }
-            (
-                PrimitiveType::Decimal {
-                    precision: _,
-                    scale: _,
-                },
-                Statistics::ByteArray(stat),
-            ) => {
-                let convert_func = |v: ByteArray| -> Result<Datum> {
-                    Result::<Datum>::Ok(Datum::new(
-                        ty.clone(),
-                        PrimitiveLiteral::Int128(i128::from_le_bytes(v.data().try_into().unwrap())),
-                    ))
-                };
-                self.update_state::<ByteArrayType>(field_id, &stat, convert_func)
-            }
-            (
-                PrimitiveType::Decimal {
-                    precision: _,
-                    scale: _,
-                },
-                Statistics::Int32(stat),
-            ) => {
-                let convert_func = |v: i32| {
-                    Result::<Datum>::Ok(Datum::new(
-                        ty.clone(),
-                        PrimitiveLiteral::Int128(i128::from(v)),
-                    ))
-                };
-                self.update_state::<Int32Type>(field_id, &stat, convert_func)
-            }
-            (
-                PrimitiveType::Decimal {
-                    precision: _,
-                    scale: _,
-                },
-                Statistics::Int64(stat),
-            ) => {
-                let convert_func = |v: i64| {
-                    Result::<Datum>::Ok(Datum::new(
-                        ty.clone(),
-                        PrimitiveLiteral::Int128(i128::from(v)),
-                    ))
-                };
-                self.update_state::<Int64Type>(field_id, &stat, convert_func)
-            }
-            (PrimitiveType::Uuid, Statistics::FixedLenByteArray(stat)) => {
-                let convert_func = |v: FixedLenByteArray| {
-                    if v.len() != 16 {
-                        return Err(Error::new(
-                            ErrorKind::Unexpected,
-                            "Invalid length of uuid bytes.",
-                        ));
-                    }
-                    Ok(Datum::uuid(Uuid::from_bytes(
-                        v.data()[..16].try_into().unwrap(),
-                    )))
-                };
-                self.update_state::<FixedLenByteArrayType>(field_id, &stat, convert_func)
-            }
-            (PrimitiveType::Fixed(len), Statistics::FixedLenByteArray(stat)) => {
-                let convert_func = |v: FixedLenByteArray| {
-                    if v.len() != *len as usize {
-                        return Err(Error::new(
-                            ErrorKind::Unexpected,
-                            "Invalid length of fixed bytes.",
-                        ));
-                    }
-                    Ok(Datum::fixed(v.data().to_vec()))
-                };
-                self.update_state::<FixedLenByteArrayType>(field_id, &stat, convert_func)
-            }
-            (ty, value) => {
+        if value.min_is_exact() {
+            let Some(min_datum) = get_parquet_stat_min_as_datum(&ty, &value)? else {
                 return Err(Error::new(
                     ErrorKind::Unexpected,
                     format!("Statistics {} is not match with field type {}.", value, ty),
-                ))
-            }
+                ));
+            };
+
+            self.update_state_min(field_id, min_datum);
         }
+
+        if value.max_is_exact() {
+            let Some(max_datum) = get_parquet_stat_max_as_datum(&ty, &value)? else {
+                return Err(Error::new(
+                    ErrorKind::Unexpected,
+                    format!("Statistics {} is not match with field type {}.", value, ty),
+                ));
+            };
+
+            self.update_state_max(field_id, max_datum);
+        }
+
         Ok(())
     }
 
@@ -498,7 +381,7 @@ impl ParquetWriter {
             // # TODO(#417)
             // - nan_value_counts
             // - distinct_counts
-            .key_metadata(metadata.footer_signing_key_metadata.unwrap_or_default())
+            .key_metadata(metadata.footer_signing_key_metadata)
             .split_offsets(
                 metadata
                     .row_groups
@@ -595,12 +478,13 @@ mod tests {
     use anyhow::Result;
     use arrow_array::types::Int64Type;
     use arrow_array::{
-        ArrayRef, BooleanArray, Int32Array, Int64Array, ListArray, RecordBatch, StructArray,
+        Array, ArrayRef, BooleanArray, Int32Array, Int64Array, ListArray, RecordBatch, StructArray,
     };
     use arrow_schema::{DataType, SchemaRef as ArrowSchemaRef};
     use arrow_select::concat::concat_batches;
     use parquet::arrow::PARQUET_FIELD_ID_META_KEY;
     use tempfile::TempDir;
+    use uuid::Uuid;
 
     use super::*;
     use crate::io::FileIOBuilder;
@@ -632,6 +516,18 @@ mod tests {
                 .into(),
                 NestedField::optional(
                     11,
+                    "timestamp_ns",
+                    Type::Primitive(PrimitiveType::TimestampNs),
+                )
+                .into(),
+                NestedField::optional(
+                    12,
+                    "timestamptz_ns",
+                    Type::Primitive(PrimitiveType::TimestamptzNs),
+                )
+                .into(),
+                NestedField::optional(
+                    13,
                     "decimal",
                     Type::Primitive(PrimitiveType::Decimal {
                         precision: 10,
@@ -639,9 +535,20 @@ mod tests {
                     }),
                 )
                 .into(),
-                NestedField::optional(12, "uuid", Type::Primitive(PrimitiveType::Uuid)).into(),
-                NestedField::optional(13, "fixed", Type::Primitive(PrimitiveType::Fixed(10)))
+                NestedField::optional(14, "uuid", Type::Primitive(PrimitiveType::Uuid)).into(),
+                NestedField::optional(15, "fixed", Type::Primitive(PrimitiveType::Fixed(10)))
                     .into(),
+                // Parquet Statistics will use different representation for Decimal with precision 38 and scale 5,
+                // so we need to add a new field for it.
+                NestedField::optional(
+                    16,
+                    "decimal_38",
+                    Type::Primitive(PrimitiveType::Decimal {
+                        precision: 38,
+                        scale: 5,
+                    }),
+                )
+                .into(),
             ])
             .build()
             .unwrap()
@@ -738,7 +645,7 @@ mod tests {
     async fn test_parquet_writer() -> Result<()> {
         let temp_dir = TempDir::new().unwrap();
         let file_io = FileIOBuilder::new_fs_io().build().unwrap();
-        let loccation_gen =
+        let location_gen =
             MockLocationGenerator::new(temp_dir.path().to_str().unwrap().to_string());
         let file_name_gen =
             DefaultFileNameGenerator::new("test".to_string(), None, DataFileFormat::Parquet);
@@ -762,7 +669,7 @@ mod tests {
             WriterProperties::builder().build(),
             Arc::new(to_write.schema().as_ref().try_into().unwrap()),
             file_io.clone(),
-            loccation_gen,
+            location_gen,
             file_name_gen,
         )
         .build()
@@ -1091,12 +998,22 @@ mod tests {
             arrow_array::TimestampMicrosecondArray::from(vec![Some(0), Some(1), None, Some(3)])
                 .with_timezone_utc(),
         ) as ArrayRef;
-        let col11 = Arc::new(
+        let col11 = Arc::new(arrow_array::TimestampNanosecondArray::from(vec![
+            Some(0),
+            Some(1),
+            None,
+            Some(3),
+        ])) as ArrayRef;
+        let col12 = Arc::new(
+            arrow_array::TimestampNanosecondArray::from(vec![Some(0), Some(1), None, Some(3)])
+                .with_timezone_utc(),
+        ) as ArrayRef;
+        let col13 = Arc::new(
             arrow_array::Decimal128Array::from(vec![Some(1), Some(2), None, Some(100)])
                 .with_precision_and_scale(10, 5)
                 .unwrap(),
         ) as ArrayRef;
-        let col12 = Arc::new(
+        let col14 = Arc::new(
             arrow_array::FixedSizeBinaryArray::try_from_sparse_iter_with_size(
                 vec![
                     Some(Uuid::from_u128(0).as_bytes().to_vec()),
@@ -1109,7 +1026,7 @@ mod tests {
             )
             .unwrap(),
         ) as ArrayRef;
-        let col13 = Arc::new(
+        let col15 = Arc::new(
             arrow_array::FixedSizeBinaryArray::try_from_sparse_iter_with_size(
                 vec![
                     Some(vec![1, 2, 3, 4, 5, 6, 7, 8, 9, 10]),
@@ -1122,8 +1039,14 @@ mod tests {
             )
             .unwrap(),
         ) as ArrayRef;
+        let col16 = Arc::new(
+            arrow_array::Decimal128Array::from(vec![Some(1), Some(2), None, Some(100)])
+                .with_precision_and_scale(38, 5)
+                .unwrap(),
+        ) as ArrayRef;
         let to_write = RecordBatch::try_new(arrow_schema.clone(), vec![
             col0, col1, col2, col3, col4, col5, col6, col7, col8, col9, col10, col11, col12, col13,
+            col14, col15, col16,
         ])
         .unwrap();
 
@@ -1171,8 +1094,10 @@ mod tests {
                 (8, Datum::time_micros(0).unwrap()),
                 (9, Datum::timestamp_micros(0)),
                 (10, Datum::timestamptz_micros(0)),
+                (11, Datum::timestamp_nanos(0)),
+                (12, Datum::timestamptz_nanos(0)),
                 (
-                    11,
+                    13,
                     Datum::new(
                         PrimitiveType::Decimal {
                             precision: 10,
@@ -1181,10 +1106,18 @@ mod tests {
                         PrimitiveLiteral::Int128(1)
                     )
                 ),
-                (12, Datum::uuid(Uuid::from_u128(0))),
-                (13, Datum::fixed(vec![1, 2, 3, 4, 5, 6, 7, 8, 9, 10])),
-                (12, Datum::uuid(Uuid::from_u128(0))),
-                (13, Datum::fixed(vec![1, 2, 3, 4, 5, 6, 7, 8, 9, 10])),
+                (14, Datum::uuid(Uuid::from_u128(0))),
+                (15, Datum::fixed(vec![1, 2, 3, 4, 5, 6, 7, 8, 9, 10])),
+                (
+                    16,
+                    Datum::new(
+                        PrimitiveType::Decimal {
+                            precision: 38,
+                            scale: 5
+                        },
+                        PrimitiveLiteral::Int128(1)
+                    )
+                ),
             ])
         );
         assert_eq!(
@@ -1201,8 +1134,10 @@ mod tests {
                 (8, Datum::time_micros(3).unwrap()),
                 (9, Datum::timestamp_micros(3)),
                 (10, Datum::timestamptz_micros(3)),
+                (11, Datum::timestamp_nanos(3)),
+                (12, Datum::timestamptz_nanos(3)),
                 (
-                    11,
+                    13,
                     Datum::new(
                         PrimitiveType::Decimal {
                             precision: 10,
@@ -1211,11 +1146,21 @@ mod tests {
                         PrimitiveLiteral::Int128(100)
                     )
                 ),
-                (12, Datum::uuid(Uuid::from_u128(3))),
+                (14, Datum::uuid(Uuid::from_u128(3))),
                 (
-                    13,
+                    15,
                     Datum::fixed(vec![21, 22, 23, 24, 25, 26, 27, 28, 29, 30])
                 ),
+                (
+                    16,
+                    Datum::new(
+                        PrimitiveType::Decimal {
+                            precision: 38,
+                            scale: 5
+                        },
+                        PrimitiveLiteral::Int128(100)
+                    )
+                ),
             ])
         );
 
